package net.sf.doolin.gui.action;

import javax.swing.Icon;

import net.sf.doolin.gui.action.support.ActionContextDataExtractor;
import net.sf.doolin.gui.action.support.DefaultActionContextDataExtractor;
import net.sf.doolin.gui.display.DisplayStateHandler;
import net.sf.doolin.gui.display.DisplayStateHandlers;
import net.sf.doolin.gui.service.IconService;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.gui.service.InfoService;
import net.sf.doolin.gui.service.icon.DefaultIconService;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * Abstract action for a default controlled behaviour.
 * 
 * @author Damien Coraboeuf
 */
public abstract class AbstractGUIAction implements GUIAction {

	private IconService iconService = new DefaultIconService();

	private DisplayStateHandler displayStateHandler = DisplayStateHandlers.ENABLED;
	private ActionContextDataExtractor actionContextDataExtractor = new DefaultActionContextDataExtractor();

	/**
	 * Executes the action and throws any kind of error.
	 * 
	 * @param context
	 *            the context
	 * 
	 * @throws Exception
	 *             the exception
	 */
	protected abstract void doExecute(ActionContext context) throws Exception;

	/**
	 * Executes the action and deals with errors.
	 * 
	 * {@inheritDoc}
	 * 
	 * @param context
	 *            the context
	 */
	@Override
	public boolean execute(ActionContext context) {
		try {
			doExecute(context);
			return true;
		} catch (Exception e) {
			InfoService logService = context.getApplication().getInfoService();
			logService.error(context, e, "GUIAction.error", getId());
			return false;
		}
	}

	/**
	 * Extracts the data to use from the action context.
	 * 
	 * @param actionContext
	 *            Action context to get the data from
	 * @return Data
	 * @see #setActionContextDataExtractor(ActionContextDataExtractor)
	 */
	protected Object getActionContextData(ActionContext actionContext) {
		return this.actionContextDataExtractor.getData(actionContext);
	}

	public ActionContextDataExtractor getActionContextDataExtractor() {
		return this.actionContextDataExtractor;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public DisplayStateHandler getDisplayStateHandler() {
		return this.displayStateHandler;
	}

	@Override
	public Icon getIcon(String iconID, IconSize iconSize) {
		if (StringUtils.isNotBlank(iconID)) {
			return this.iconService.getIcon(iconID, iconSize);
		} else {
			return null;
		}
	}

	/**
	 * Gets the icon service.
	 * 
	 * @return the icon service
	 */
	public IconService getIconService() {
		return this.iconService;
	}

	public void setActionContextDataExtractor(
			ActionContextDataExtractor actionContextDataExtractor) {
		this.actionContextDataExtractor = actionContextDataExtractor;
	}

	/**
	 * Sets the display state handler.
	 * 
	 * @param displayStateHandler
	 *            the new display state handler
	 */
	public void setDisplayStateHandler(DisplayStateHandler displayStateHandler) {
		this.displayStateHandler = displayStateHandler;
	}

	/**
	 * Sets the icon service.
	 * 
	 * @param iconService
	 *            the new icon service
	 */
	@Autowired(required = false)
	public void setIconService(IconService iconService) {
		this.iconService = iconService;
	}

}
