package net.sf.doolin.gui.action;

import javax.swing.JComponent;

import net.sf.doolin.bus.SubscriberValidator;
import net.sf.doolin.gui.app.GUIApplication;
import net.sf.doolin.gui.validation.ValidationSupport;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.window.GUIWindow;

/**
 * Context for the execution of an action.
 * <p>
 * This interface is central to the Doolin GUI framework. It allows almost each
 * element of an application (mainly {@link GUIAction} instances) to know
 * <i>where</i> they are running: the application, the window, the view, the
 * data and the contextual data.
 * </p>
 * 
 * @author Damien Coraboeuf
 */
public interface ActionContext {

	/**
	 * Gets the running application
	 * 
	 * @return Application (never <code>null</code>)
	 */
	GUIApplication getApplication();

	/**
	 * Gets the component associated with this context.
	 * 
	 * @return Component
	 */
	JComponent getComponent();

	/**
	 * Gets any contextual information linked to this context. While the
	 * <i>context data</i> cannot change (only its content), the <i>contextual
	 * data</i> can change at any time.
	 * 
	 * @return Contextual information or <code>null</code> if none has been
	 *         defined.
	 */
	Object getContext();

	/**
	 * Gets the data associated with this context.
	 * 
	 * @return Model
	 */
	Object getData();

	/**
	 * Gets the validator to use for any subscription managed by this context.
	 * The returned validator will {@linkplain SubscriberValidator#isValid()
	 * return} <code>false</code> as soon as this {@link ActionContext} is no
	 * longer valid.
	 * 
	 * @return Validator
	 */
	SubscriberValidator getSubscriberValidator();

	/**
	 * Gets the validation support for this action context
	 * 
	 * @param <V>
	 *            Type of the bean that is managed by the validation support. It
	 *            should be the same than the one associated with the window or
	 *            the view.
	 * @return Validation support
	 */
	<V> ValidationSupport<V> getValidationSupport();

	/**
	 * Gets the view associated to this content
	 * 
	 * @param <T>
	 *            Type of model for the view
	 * @return View or <code>null</code> if no view is associated to this
	 *         context
	 */
	<T> GUIView<T> getView();

	/**
	 * Gets the window associated to this content
	 * 
	 * @param <T>
	 *            Type of model for the window
	 * @return Window or <code>null</code> is no window can be associated with
	 *         this context.
	 */
	<T> GUIWindow<T> getWindow();

	/**
	 * Sets the contextual value of this context.
	 * 
	 * @param value
	 *            New contextual value
	 */
	void setContext(Object value);

}
