package net.sf.doolin.gui.bus;

import java.util.List;

import net.sf.doolin.bus.Bus;
import net.sf.doolin.bus.BusLog;
import net.sf.doolin.bus.IBusInfo;
import net.sf.doolin.bus.ISubscriptionClassInfo;
import net.sf.doolin.bus.ISubscriptionInfo;
import net.sf.doolin.bus.bean.Bean;

import org.joda.time.DateTime;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.util.concurrent.Lock;

/**
 * Bean model used by the Bus Monitoring API. It contains a list of
 * {@link BusMonitoringSubscription} and manages the count of them.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class BusMonitoringModel extends Bean {

	public static final String AUTO_REFRESH = "autoRefresh";
	public static final String REFRESH_INTERVAL = "refreshInterval";
	public static final String MAXIMUM = "maximum";
	public static final String COUNT = "count";
	public static final String ACTIVATE_LOGGING = "activateLogging";

	private static final long serialVersionUID = 1L;

	private final EventList<BusMonitoringSubscription> subscriptionList = new BasicEventList<BusMonitoringSubscription>();
	private final EventList<BusMonitoringStat> statList = new BasicEventList<BusMonitoringStat>();

	private int count;
	private int maximum;
	private int refreshInterval = 5;
	private boolean autoRefresh;
	private boolean activateLogging;

	private final BusLog defaultBusLog;

	private final BusMonitoringLog monitoringLog;

	/**
	 * Constructor.
	 * 
	 * Starts to build the list of subscriptions.
	 * 
	 * @see #refresh()
	 */
	public BusMonitoringModel() {
		this.defaultBusLog = Bus.get().getLog();
		this.monitoringLog = new BusMonitoringLog(this.defaultBusLog);
		refresh();
	}

	public void clearLog() {
		this.monitoringLog.clear();
	}

	/**
	 * Returns the count of subscriptions.
	 * 
	 * @return Count of subscriptions
	 */
	public int getCount() {
		return this.count;
	}

	public int getMaximum() {
		return this.maximum;
	}

	public BusMonitoringLog getMonitoringLog() {
		return this.monitoringLog;
	}

	public int getRefreshInterval() {
		return this.refreshInterval;
	}

	public EventList<BusMonitoringStat> getStatList() {
		return this.statList;
	}

	/**
	 * Returns the dynamic list of subscriptions
	 * 
	 * @return List of subscriptions
	 */
	public EventList<BusMonitoringSubscription> getSubscriptionList() {
		return this.subscriptionList;
	}

	public boolean isActivateLogging() {
		return this.activateLogging;
	}

	public boolean isAutoRefresh() {
		return this.autoRefresh;
	}

	/**
	 * Refreshes the list of subscriptions from the current bus state
	 */
	public void refresh() {
		Lock lock = this.subscriptionList.getReadWriteLock().writeLock();
		lock.lock();
		try {
			this.subscriptionList.clear();
			// Gets information from the bus
			IBusInfo busInfo = Bus.get().getBusInfo();
			// TODO General information about the bus
			// Gets the list of subscriptions from the bus
			List<ISubscriptionClassInfo> subscriptionClassInfos = busInfo.getSubscriptionClassInfos();
			int id = 0;
			for (ISubscriptionClassInfo subscriptionClassInfo : subscriptionClassInfos) {
				String messageClassName = subscriptionClassInfo.getMessageClassName();
				List<ISubscriptionInfo> subscriptionInfos = subscriptionClassInfo.getSubscriptionInfos();
				for (ISubscriptionInfo subscriptionInfo : subscriptionInfos) {
					BusMonitoringSubscription subscription = new BusMonitoringSubscription(
							++id, messageClassName, subscriptionInfo);
					this.subscriptionList.add(subscription);					
				}
			}
		} finally {
			lock.unlock();
		}
		setCount(this.subscriptionList.size());
	}

	public void setActivateLogging(boolean activateLogging) {
		notify(ACTIVATE_LOGGING, this.activateLogging,
				this.activateLogging = activateLogging);
		// Activates the logging on the bus
		if (activateLogging) {
			Bus.get().setLog(this.monitoringLog);
		} else {
			Bus.get().setLog(this.defaultBusLog);
		}
	}

	public void setAutoRefresh(boolean autoRefresh) {
		notify(AUTO_REFRESH, this.autoRefresh, this.autoRefresh = autoRefresh);
	}

	/**
	 * Sets the count of subscriptions
	 * 
	 * @param count
	 *            Count
	 */
	protected void setCount(int count) {
		int max = Math.max(this.maximum, count);
		if (max > this.maximum) {
			setMaximum(max);
		}
		// Notification of the count
		notify(COUNT, this.count, this.count = count);
		// Update of the stat list
		updateStat();
	}

	/**
	 * Sets the maximum count
	 */
	protected void setMaximum(int maximum) {
		notify(MAXIMUM, this.maximum, this.maximum = maximum);
	}

	public void setRefreshInterval(int refreshInterval) {
		notify(REFRESH_INTERVAL, this.refreshInterval,
				this.refreshInterval = refreshInterval);
	}

	protected void updateStat() {
		BusMonitoringStat stat = new BusMonitoringStat(new DateTime(),
				this.count, this.maximum);
		Lock lock = this.statList.getReadWriteLock().writeLock();
		lock.lock();
		try {
			this.statList.add(stat);
		} finally {
			lock.unlock();
		}
	}

}
