package net.sf.doolin.gui.bus;

import java.lang.ref.WeakReference;

import net.sf.doolin.bus.Bus;
import net.sf.doolin.bus.ISubscriptionInfo;
import net.sf.doolin.bus.Subscriber;

import org.joda.time.DateTime;

/**
 * Bean model for the representation of a {@link Bus} subscription.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class BusMonitoringSubscription {

	private final int id;
	private final String trigger;
	private final String execution;
	private final String validator;
	private final String description;
	private final String type;

	private final WeakReference<ISubscriptionInfo> subscriber;
	private final DateTime registrationTimestamp;
	private final long registrationPeriod;

	/**
	 * Constructor
	 * 
	 * @param id
	 *            ID for this subscription
	 * @param type Type of message
	 * @param subscriberInfo
	 *            Associated subscriber information
	 */
	public BusMonitoringSubscription(int id, String type, ISubscriptionInfo subscriberInfo) {
		this.id = id;
		this.type = type;
		this.trigger = subscriberInfo.getTrigger();
		this.execution = subscriberInfo.getExecution();
		this.validator = subscriberInfo.getValidator();
		this.description = subscriberInfo.getDescription();
		// Registration timestamp
		long registrationTime = subscriberInfo.getRegistrationTime();
		this.registrationTimestamp = new DateTime(registrationTime);
		// Registration duration
		this.registrationPeriod = (System.currentTimeMillis() - registrationTime);
		// Subscriber reference
		this.subscriber = new WeakReference<ISubscriptionInfo>(subscriberInfo);
	}

	public String getExecution() {
		return this.execution;
	}

	/**
	 * Returns the ID of this subscription
	 * 
	 * @return ID of this subscription
	 */
	public int getId() {
		return this.id;
	}

	public long getRegistrationPeriod() {
		return this.registrationPeriod;
	}

	public DateTime getRegistrationTimestamp() {
		return this.registrationTimestamp;
	}

	public String getTrigger() {
		return this.trigger;
	}

	/**
	 * Returns the type of this subscription
	 * 
	 * @return Type of this subscription
	 */
	public String getType() {
		return this.type;
	}

	public String getValidator() {
		return this.validator;
	}
	
	public String getDescription() {
		return description;
	}

	/**
	 * Tests if the associated subscription is still valid.
	 * 
	 * @return <code>true</code> if the subscriber is still available and still
	 *         {@linkplain Subscriber#isValid() valid}.
	 */
	public boolean test() {
		ISubscriptionInfo s = this.subscriber.get();
		if (s != null) {
			return s.isValid();
		} else {
			return false;
		}
	}

	@Override
	public String toString() {
		return String.format("%s -> %s (%s)", this.trigger, this.execution,
				this.validator);
	}

}
