package net.sf.doolin.gui.display;

import net.sf.doolin.bus.SubscriberValidator;
import net.sf.doolin.bus.support.AndSubscriberValidator;
import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.util.Utils;

/**
 * This display state handler is based on a property of the context. This
 * property is adapted to a display state using an adapter.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <S>
 *            Type of the property to convert to a display state
 */
public abstract class AbstractContextPropertyDisplayStateHandler<S> extends
		AbstractDisplayStateHandler {

	private String propertyPath = "this";

	/**
	 * Default constructor
	 */
	public AbstractContextPropertyDisplayStateHandler() {
	}

	/**
	 * Constructor.
	 * 
	 * @param propertyPath
	 *            Path to the property of the context to take into account
	 */
	public AbstractContextPropertyDisplayStateHandler(String propertyPath) {
		this.propertyPath = propertyPath;
	}

	@SuppressWarnings("unchecked")
	@Override
	public DisplayState getDisplayState(ActionContext actionContext) {
		Object context = actionContext.getContext();
		S property = null;
		if (context != null) {
			property = (S) Utils.getProperty(context, this.propertyPath);
		}
		return getDisplayState(property);
	}

	/**
	 * Computes the display state from the given property value
	 * 
	 * @param property
	 *            Property value
	 * @return Display state
	 */
	protected abstract DisplayState getDisplayState(S property);

	/**
	 * Gets the property path.
	 * 
	 * @return the property path
	 */
	public String getPropertyPath() {
		return this.propertyPath;
	}

	/**
	 * Sets the property path.
	 * 
	 * @param propertyPath
	 *            the new property path
	 */
	public void setPropertyPath(String propertyPath) {
		this.propertyPath = propertyPath;
	}

	@Override
	public void subscribe(final ActionContext actionContext, final Runnable task) {
		Runnable contextSubscription = new Runnable() {

			@Override
			public void run() {
				final Object context = actionContext.getContext();
				if (context != null) {
					// Validator based on the current value of the content
					AndSubscriberValidator validator = new AndSubscriberValidator(
							new SubscriberValidator() {

								@Override
								public String getValidatorDescription() {
									return "Context of " + actionContext
											+ " == " + context;
								}

								@Override
								public boolean isValid() {
									return (context == actionContext
											.getContext());
								}
							}, actionContext.getSubscriberValidator());
					// Subscribes to changes on the current context
					PropertyChangeSupport
							.subscribe(
									validator,
									context,
									AbstractContextPropertyDisplayStateHandler.this.propertyPath,
									task);
				}
			}
		};
		// Initial subscription
		contextSubscription.run();
		// Context change
		PropertyChangeSupport.subscribe(actionContext.getSubscriberValidator(),
				actionContext, "context", contextSubscription);
	}

}
