package net.sf.doolin.gui.display;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;

/**
 * Operator that evaluates to boolean results.
 * 
 * @author Damien Coraboeuf
 * 
 */
public enum ConditionalOperator {

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is not
	 * <code>null</code>. The operand is not used.
	 */
	NOT_NULL {
		@Override
		public boolean evaluate(Object value, Object operand) {
			return (value != null);
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is *
	 * <code>null</code>. The operand is not used.
	 */
	NULL {
		@Override
		public boolean evaluate(Object value, Object operand) {
			return (value == null);
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> as a string is not
	 * blank. The operand is not used.
	 */
	NOT_BLANK {
		@Override
		public boolean evaluate(Object value, Object operand) {
			return StringUtils.isNotBlank(ObjectUtils.toString(value, null));
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> as a string is
	 * blank. The operand is not used.
	 */
	BLANK {
		@Override
		public boolean evaluate(Object value, Object operand) {
			return StringUtils.isBlank(ObjectUtils.toString(value, null));
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is equal to the
	 * <code>operand</code>.
	 */
	EQ {
		@Override
		public boolean evaluate(Object value, Object operand) {
			return ObjectUtils.equals(value, operand);
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is equal to the
	 * <code>operand</code> when comparing their String representation.
	 */
	EQ_STR {
		@Override
		public boolean evaluate(Object value, Object operand) {
			return StringUtils.equals(ObjectUtils.toString(value), ObjectUtils
					.toString(operand));
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is not equal to the
	 * <code>operand</code>.
	 */
	NEQ {
		@Override
		public boolean evaluate(Object value, Object operand) {
			return !ObjectUtils.equals(value, operand);
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is greater or equal
	 * than the <code>operand</code>.
	 */
	GTE {
		@Override
		@SuppressWarnings("unchecked")
		public boolean evaluate(Object value, Object operand) {
			return operand != null
					&& value instanceof Comparable
					&& ((Comparable) value).compareTo(operand) >= 0;
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is greater than the
	 * <code>operand</code>.
	 */
	GT {
		@Override
		@SuppressWarnings("unchecked")
		public boolean evaluate(Object value, Object operand) {
			return operand != null
					&& value instanceof Comparable
					&& ((Comparable) value).compareTo(operand) > 0;
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is less or equal
	 * than the <code>operand</code>.
	 */
	LTE {
		@Override
		@SuppressWarnings("unchecked")
		public boolean evaluate(Object value, Object operand) {
			return operand != null
					&& value instanceof Comparable
					&& ((Comparable) value).compareTo(operand) <= 0;
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> is less than the
	 * <code>operand</code>.
	 */
	LT {
		@Override
		@SuppressWarnings("unchecked")
		public boolean evaluate(Object value, Object operand) {
			return operand != null
					&& value instanceof Comparable
					&& ((Comparable) value).compareTo(operand) < 0;
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> can be evaluated to
	 * a boolean that is <code>true</code>.
	 */
	BOOLEAN_TRUE {
		@Override
		public boolean evaluate(Object value, Object operand) {
			if (value instanceof Boolean) {
				return ((Boolean) value);
			} else {
				String stringValue = ObjectUtils.toString(value, null);
				if (stringValue == null) {
					return false;
				} else {
					return Boolean.valueOf(stringValue);
				}
			}
		}
	},

	/**
	 * Evaluates to <code>true</code> if <code>value</code> can be evaluated to
	 * a boolean that is <code>false</code>.
	 */
	BOOLEAN_FALSE {
		@Override
		public boolean evaluate(Object value, Object operand) {
			if (value instanceof Boolean) {
				return !((Boolean) value);
			} else {
				String stringValue = ObjectUtils.toString(value, null);
				if (stringValue == null) {
					return true;
				} else {
					return !Boolean.valueOf(stringValue);
				}
			}
		}
	};

	/**
	 * Evaluates the operator with the value and the operand.
	 * 
	 * @param value
	 *            Value
	 * @param operand
	 *            Operand (may not be used by some operators)
	 * @return Result of the evaluation
	 */
	public abstract boolean evaluate(Object value, Object operand);

}
