package net.sf.doolin.gui.edit;

import java.util.Stack;

import net.sf.doolin.bus.Bus;

/**
 * Default implementation for the {@link GUIEditManager}. It maintains a dual
 * list of done and undone tasks.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class DefaultGUIEditManager implements GUIEditManager {

	private final Stack<GUIEdit> doneEdits = new Stack<GUIEdit>();
	private final Stack<GUIEdit> undoneEdits = new Stack<GUIEdit>();

	@Override
	public synchronized void addEdit(GUIEdit record) {
		this.doneEdits.add(record);
		this.undoneEdits.clear();
		fireUpdated();
	}

	@Override
	public boolean canRedo() {
		return !this.undoneEdits.isEmpty();
	}

	@Override
	public boolean canUndo() {
		return !this.doneEdits.isEmpty();
	}

	@Override
	public String getUndoDisplayName() {
		if (this.doneEdits.isEmpty()) {
			return null;
		} else {
			GUIEdit edit = this.doneEdits.peek();
			return edit.getDisplayName();
		}
	}

	@Override
	public String getRedoDisplayName() {
		if (this.undoneEdits.isEmpty()) {
			return null;
		} else {
			GUIEdit edit = this.undoneEdits.peek();
			return edit.getDisplayName();
		}
	}

	@Override
	public void undo() {
		if (this.doneEdits.isEmpty()) {
			throw new IllegalStateException("Cannot undo");
		} else {
			GUIEdit edit = this.doneEdits.peek();
			try {
				edit.undo();
			} finally {
				this.doneEdits.pop();
				this.undoneEdits.push(edit);
				fireUpdated();
			}
		}
	}

	@Override
	public void redo() {
		if (this.undoneEdits.isEmpty()) {
			throw new IllegalStateException("Cannot redo");
		} else {
			GUIEdit edit = this.undoneEdits.peek();
			try {
				edit.redo();
			} finally {
				this.undoneEdits.pop();
				this.doneEdits.push(edit);
				fireUpdated();
			}
		}
	}

	@Override
	public void clear() {
		this.undoneEdits.clear();
		this.doneEdits.clear();
		fireUpdated();
	}

	/**
	 * Publishes to the bus that the state of tasks has changed
	 * 
	 * @see GUIEditManagerUpdateMessage
	 */
	protected void fireUpdated() {
		Bus.get().publish(new GUIEditManagerUpdateMessage(this));
	}

}
