package net.sf.doolin.gui.field;

import javax.swing.JComponent;
import javax.swing.TransferHandler;

import net.sf.doolin.bus.bean.Bean;
import net.sf.doolin.gui.action.swing.ActionFactory;
import net.sf.doolin.gui.action.swing.DefaultActionFactory;
import net.sf.doolin.gui.display.DisplayState;
import net.sf.doolin.gui.display.DisplayStateHandler;
import net.sf.doolin.gui.display.DisplayStateHandlers;
import net.sf.doolin.gui.service.IconService;
import net.sf.doolin.gui.service.icon.DefaultIconService;
import net.sf.doolin.gui.style.Style;

import org.springframework.beans.factory.annotation.Autowired;

/**
 * Abstract field descriptor.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 */
public abstract class AbstractFieldDescriptor<V> extends Bean implements
		FieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private String constraint;

	private String name;

	private DisplayStateHandler displayStateHandler = DisplayStateHandlers.ENABLED;

	private ActionFactory actionFactory = DefaultActionFactory.INSTANCE;

	private Style style;

	private IconService iconService = new DefaultIconService();

	private TransferHandler transferHandler;

	public ActionFactory getActionFactory() {
		return this.actionFactory;
	}

	/**
	 * Gets the constraint.
	 * 
	 * @return the constraint
	 */
	public String getConstraint() {
		return this.constraint;
	}

	public DisplayStateHandler getDisplayStateHandler() {
		return this.displayStateHandler;
	}

	/**
	 * Gets the {@link IconService} service that is used to access icons.
	 * 
	 * @return Icon service
	 */
	public IconService getIconService() {
		return this.iconService;
	}

	public String getName() {
		return this.name;
	}

	/**
	 * Returns the style
	 * 
	 * @return Style
	 */
	public Style getStyle() {
		return this.style;
	}

	public TransferHandler getTransferHandler() {
		return this.transferHandler;
	}

	/**
	 * Does nothing.
	 */
	@Override
	public void onInit(Field<V> field) {
	}

	/**
	 * Does nothing.
	 */
	@Override
	public void onShow(Field<V> field) {
	}

	/**
	 * Sets the action factory for the creation of action-related components.
	 * 
	 * @param actionFactory
	 *            Action factory
	 */
	@Autowired(required = false)
	public void setActionFactory(ActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}

	/**
	 * Sets the constraint for the layout.
	 * 
	 * @param constraint
	 *            the new constraint
	 */
	public void setConstraint(String constraint) {
		this.constraint = constraint;
	}

	@Override
	public void setDisplayState(Field<V> field, JComponent component,
			DisplayState displayState) {
		JComponent fieldComponent = field.getFieldComponent();
		switch (displayState) {
		case ENABLED:
			fieldComponent.setEnabled(true);
			component.setVisible(true);
			break;
		case DISABLED:
			fieldComponent.setEnabled(false);
			component.setVisible(true);
			break;
		case HIDDEN:
			fieldComponent.setEnabled(false);
			component.setVisible(false);
			break;
		}
	}

	/**
	 * Sets the display state handler.
	 * 
	 * @param displayStateHandler
	 *            the new display state handler
	 */
	public void setDisplayStateHandler(DisplayStateHandler displayStateHandler) {
		this.displayStateHandler = displayStateHandler;
	}

	/**
	 * Sets the {@link IconService} service that is used to access the icons.
	 * 
	 * <p>
	 * <b>Note:</b>It is {@link Autowired} by the Spring configuration.
	 * 
	 * @param iconService
	 *            Icon service
	 */
	@Autowired(required = false)
	public void setIconService(IconService iconService) {
		this.iconService = iconService;
	}

	/**
	 * Sets the name for the field.
	 * 
	 * @param name
	 *            Field name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Sets the style for the component
	 * 
	 * @param style
	 *            Style
	 */
	public void setStyle(Style style) {
		this.style = style;
	}

	/**
	 * Sets the transfer handler for this field.
	 * 
	 * @param transferHandler
	 *            Transfer handler
	 */
	public void setTransferHandler(TransferHandler transferHandler) {
		this.transferHandler = transferHandler;
	}

}
