package net.sf.doolin.gui.field;

import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

import net.sf.doolin.gui.display.DisplayState;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.view.GUIView;

/**
 * {@link JTextField} field descriptor.
 * 
 * @param <V>
 *            Type of object for the view
 */
public abstract class AbstractFieldText<V, J extends JTextComponent> extends AbstractPropertyFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private boolean commitOnFocusLost = false;

	private int columns = 10;

	/**
	 * {@inheritDoc}.
	 * 
	 * @param view
	 *            the view
	 * 
	 * @return the field< v>
	 */
	@Override
	public Field<V> createField(final GUIView<V> view) {
		// Creates the text field
		final J text = createTextComponent();

		// Flag for update of the text field
		final AtomicBoolean fieldUpdated = new AtomicBoolean(false);

		// Model -> Component
		Runnable updateComponentTask = new Runnable() {

			@Override
			public void run() {
				String s = (String) getProperty(view);
				if (!fieldUpdated.get()) {
					text.setText(s);
				}
			}
		};
		subscribe(view, updateComponentTask);

		// Initial value
		updateComponentTask.run();

		// Component -> Model
		// on Focus Lost
		if (this.commitOnFocusLost) {
			FocusAdapter focusAdapter = new FocusAdapter() {

				@Override
				public void focusLost(FocusEvent e) {
					if (text.isEditable()) {
						String s = text.getText();
						setProperty(view, s);
					}
				}

			};
			text.addFocusListener(focusAdapter);
		}
		// on Change
		else {
			Document document = text.getDocument();
			document.addDocumentListener(new DocumentListener() {

				@Override
				public void changedUpdate(DocumentEvent e) {
					update();
				}

				@Override
				public void insertUpdate(DocumentEvent e) {
					update();
				}

				@Override
				public void removeUpdate(DocumentEvent e) {
					update();
				}

				protected void update() {
					String s = text.getText();
					fieldUpdated.set(true);
					try {
						setProperty(view, s);
					} finally {
						fieldUpdated.set(false);
					}
				}
			});
		}

		// Decorates the text component
		JComponent decoratedComponent = decorateComponent(text);
		// Creates the field
		SimpleField<V> field = new SimpleField<V>(view, this, decoratedComponent);
		// Associates the non-decorated text component as a property
		field.setProperty(AbstractFieldText.class, text);
		// OK
		return field;
	}

	protected abstract J createTextComponent();

	/**
	 * Decorates the text field before it is actually set as the field component
	 * 
	 * @param text
	 *            Text component
	 * @return Actual component
	 */
	protected JComponent decorateComponent(J text) {
		return text;
	}

	/**
	 * Gets the columns.
	 * 
	 * @return the columns
	 */
	public int getColumns() {
		return this.columns;
	}

	/**
	 * Checks if is commit on focus lost.
	 * 
	 * @return true, if is commit on focus lost
	 */
	public boolean isCommitOnFocusLost() {
		return this.commitOnFocusLost;
	}

	/**
	 * Sets the columns.
	 * 
	 * @param columns
	 *            the new columns
	 */
	public void setColumns(int columns) {
		this.columns = columns;
	}

	/**
	 * Sets the commit on focus lost.
	 * 
	 * @param commitOnFocusLost
	 *            the new commit on focus lost
	 */
	public void setCommitOnFocusLost(boolean commitOnFocusLost) {
		this.commitOnFocusLost = commitOnFocusLost;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setDisplayState(Field<V> field, JComponent component, DisplayState displayState) {
		JTextComponent fieldComponent = (JTextComponent) field.getProperty(AbstractFieldText.class);
		switch (displayState) {
		case ENABLED:
			fieldComponent.setEnabled(true);
			fieldComponent.setEditable(true);
			component.setVisible(true);
			break;
		case DISABLED:
			fieldComponent.setEnabled(true);
			fieldComponent.setEditable(false);
			component.setVisible(true);
			break;
		case HIDDEN:
			fieldComponent.setEnabled(false);
			fieldComponent.setEditable(false);
			component.setVisible(false);
			break;
		}
	}

}
