package net.sf.doolin.gui.field;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import net.sf.doolin.bus.SubscriberValidator;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.swing.ComboFilePath;
import net.sf.doolin.gui.swing.FileBrowsingMode;
import net.sf.doolin.gui.swing.IFilePath;
import net.sf.doolin.gui.swing.JFile;
import net.sf.doolin.gui.swing.TextFilePath;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.factory.DataFactory;
import ca.odell.glazedlists.EventList;

/**
 * Field for a file selection.
 * 
 * @param <V>
 *            Type of object for the view
 */
public class FieldFile<V> extends AbstractPropertyFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private boolean directory = false;

	private FileBrowsingMode mode = FileBrowsingMode.OPEN;

	private String filter;

	private String filterDescription;

	private DataFactory<EventList<String>> itemListFactory;

	/**
	 * {@inheritDoc}.
	 * 
	 * @param view
	 *            the view
	 * 
	 * @return the field< v>
	 */
	@Override
	public Field<V> createField(final GUIView<V> view) {
		// Filter expressions
		V viewModel = view.getViewData();
		final GUIExpression filterExpression = new GUIExpression(view
				.getActionContext().getSubscriberValidator(), viewModel,
				this.filter);
		final GUIExpression filterDescriptionExpression = new GUIExpression(
				view.getActionContext().getSubscriberValidator(), viewModel,
				this.filterDescription);
		// File path component
		IFilePath filePath;
		if (this.itemListFactory == null) {
			filePath = new TextFilePath();
		} else {
			EventList<String> fileList = this.itemListFactory.create(viewModel);
			filePath = new ComboFilePath(fileList);
		}
		// Creates the file field
		final JFile jfile = new JFile(view.getApplication().getPreferences(),
				filePath);
		// Set-up
		jfile.setDirectory(this.directory);
		jfile.setMode(this.mode);
		jfile.setFilter(filterExpression.getValue());
		jfile.setFilterDescription(filterDescriptionExpression.getValue());
		// Listens for filter descriptions
		SubscriberValidator subscriberValidator = view.getActionContext()
				.getSubscriberValidator();
		filterExpression.connect(subscriberValidator, jfile, "filter");
		filterDescriptionExpression.connect(subscriberValidator, jfile,
				"filterDescription");
		// Initialisation
		File file = (File) getProperty(view);
		jfile.setFile(file);

		// Model -> Field and Initial value
		subscribe(view, jfile, JFile.PROPERTY_FILE);
		// Field -> Model
		jfile.addPropertyChangeListener(JFile.PROPERTY_FILE,
				new PropertyChangeListener() {

					@Override
					public void propertyChange(PropertyChangeEvent evt) {
						setProperty(view, evt.getNewValue());
					}
				});

		// OK
		return new SimpleField<V>(view, this, jfile);
	}

	/**
	 * Gets the filter.
	 * 
	 * @return the filter
	 */
	public String getFilter() {
		return this.filter;
	}

	/**
	 * Gets the filter description.
	 * 
	 * @return the filter description
	 */
	public String getFilterDescription() {
		return this.filterDescription;
	}

	/**
	 * Gets the item list factory when the user can select a path among some
	 * predefined one or enter one.
	 * 
	 * @return Factory for the list of paths
	 */
	public DataFactory<EventList<String>> getItemListFactory() {
		return this.itemListFactory;
	}

	/**
	 * Gets the mode.
	 * 
	 * @return the mode
	 */
	public FileBrowsingMode getMode() {
		return this.mode;
	}

	/**
	 * Checks if is directory.
	 * 
	 * @return true, if is directory
	 */
	public boolean isDirectory() {
		return this.directory;
	}

	/**
	 * Selection of directories
	 * 
	 * @param directory
	 *            the new directory
	 */
	public void setDirectory(boolean directory) {
		this.directory = directory;
	}

	/**
	 * Sets the filter for files.
	 * 
	 * @param filter
	 *            the new filter
	 */
	public void setFilter(String filter) {
		this.filter = filter;
	}

	/**
	 * Description of the filter for files
	 * 
	 * @param filterDescription
	 *            the new filter description
	 */
	public void setFilterDescription(String filterDescription) {
		this.filterDescription = filterDescription;
	}

	/**
	 * Sets the item list factory when the user can select a path among some
	 * predefined one or enter one.
	 * 
	 * @param itemListFactory
	 *            Factory for the list of paths
	 */
	public void setItemListFactory(
			DataFactory<EventList<String>> itemListFactory) {
		this.itemListFactory = itemListFactory;
	}

	/**
	 * Sets the file selection mode.
	 * 
	 * @param mode
	 *            the new mode
	 */
	public void setMode(FileBrowsingMode mode) {
		this.mode = mode;
	}

}
