package net.sf.doolin.gui.field;

import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.doolin.bus.SubscriberValidator;
import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;

/**
 * Field for an {@link Integer} value.
 * <p>
 * It uses a {@link JSpinner} component.
 * <p>
 * The bounds of the component are defined, by order or priority:
 * <ol>
 * <li>property path - either {@link #setMinimumProperty(String)} or
 * {@link #setMaximumProperty(String)}
 * <li>direct value - either {@link #setMinimum(Integer)} or
 * {@link #setMaximum(Integer)}
 * <li><code>null</code> (unbounded)
 * </ol>
 * 
 * @param <V>
 *            Type of object for the view
 */
public class FieldInteger<V> extends AbstractPropertyFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private Integer minimum;
	private Integer maximum;

	private String minimumProperty;
	private String maximumProperty;

	private int step = 1;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Field<V> createField(final GUIView<V> view) {

		// View data
		final V data = view.getViewData();

		// View life-cycle
		SubscriberValidator subscriberValidator = view.getActionContext()
				.getSubscriberValidator();

		// Boundaries
		Integer min, max;

		// Min
		if (StringUtils.isNotBlank(this.minimumProperty)) {
			min = (Integer) Utils.getProperty(data, this.minimumProperty);
		} else {
			min = this.minimum;
		}

		// Max
		if (StringUtils.isNotBlank(this.maximumProperty)) {
			max = (Integer) Utils.getProperty(data, this.maximumProperty);
		} else {
			max = this.maximum;
		}

		// Value
		Integer value = min != null ? min : 0;

		// Iteration field
		final SpinnerNumberModel spinnerModel = new SpinnerNumberModel(value,
				min, max, Integer.valueOf(this.step));
		JSpinner spinner = new JSpinner(spinnerModel);

		// Changes on minimum property
		if (StringUtils.isNotBlank(this.minimumProperty)) {
			PropertyChangeSupport.subscribe(subscriberValidator, data,
					this.minimumProperty, new Runnable() {

						@Override
						public void run() {
							Integer min = (Integer) Utils.getProperty(data,
									FieldInteger.this.minimumProperty);
							spinnerModel.setMinimum(min);
						}
					});
		}

		// Changes on maximum property
		if (StringUtils.isNotBlank(this.maximumProperty)) {
			PropertyChangeSupport.subscribe(subscriberValidator, data,
					this.maximumProperty, new Runnable() {

						@Override
						public void run() {
							Integer max = (Integer) Utils.getProperty(data,
									FieldInteger.this.maximumProperty);
							spinnerModel.setMaximum(max);
						}
					});
		}

		// Model -> Spinner
		PropertyChangeSupport.connectOneWayAndUpdate(subscriberValidator, view
				.getViewData(), getPropertyPath(), spinnerModel, "value");
		// Spinner -> Model
		spinnerModel.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				setProperty(view, spinnerModel.getValue());
			}
		});
		// OK
		return new SimpleField<V>(view, this, spinner);
	}

	/**
	 * Gets the maximum.
	 * 
	 * @return the maximum
	 */
	public Integer getMaximum() {
		return this.maximum;
	}

	/**
	 * Returns the path to the property that contains the maximum value for this
	 * field.
	 * 
	 * @return Property path
	 */
	public String getMaximumProperty() {
		return this.maximumProperty;
	}

	/**
	 * Gets the minimum.
	 * 
	 * @return the minimum
	 */
	public Integer getMinimum() {
		return this.minimum;
	}

	/**
	 * Returns the path to the property that contains the minimum value for this
	 * field.
	 * 
	 * @return Property path
	 */
	public String getMinimumProperty() {
		return this.minimumProperty;
	}

	/**
	 * Returns the step which is used for this field.
	 * 
	 * @return Step between each value
	 */
	public int getStep() {
		return this.step;
	}

	/**
	 * Sets the maximum.
	 * 
	 * @param maximum
	 *            the new maximum
	 */
	public void setMaximum(Integer maximum) {
		this.maximum = maximum;
	}

	/**
	 * Sets the path to the property that contains the maximum value for this
	 * field.
	 * 
	 * @param maximumProperty
	 *            Property path
	 */
	public void setMaximumProperty(String maximumProperty) {
		this.maximumProperty = maximumProperty;
	}

	/**
	 * Sets the minimum.
	 * 
	 * @param minimum
	 *            the new minimum
	 */
	public void setMinimum(Integer minimum) {
		this.minimum = minimum;
	}

	/**
	 * Sets the path to the property that contains the minimum value for this
	 * field.
	 * 
	 * @param minimumProperty
	 *            Property path
	 */
	public void setMinimumProperty(String minimumProperty) {
		this.minimumProperty = minimumProperty;
	}

	/**
	 * Sets the step which is used for this field. It is set by default to 1.
	 * 
	 * @param step
	 *            Step between each value
	 */
	public void setStep(int step) {
		this.step = step;
	}

}
