package net.sf.doolin.gui.field;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.swing.MnemonicInfo;
import net.sf.doolin.gui.view.GUIView;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;

/**
 * This field defines a {@link JRadioButton}.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of bean for the view
 * @param <T>
 *            Value associated with the radio button
 */
public class FieldRadio<V, T> extends AbstractPropertyFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private String group;
	private String text;
	private T value;

	@Override
	public Field<V> createField(final GUIView<V> view) {

		// Creates the radio button
		final JRadioButton c = new JRadioButton();

		// Group
		if (StringUtils.isNotBlank(this.group)) {
			ButtonGroup buttonGroup = (ButtonGroup) view
					.getProperty(this.group);
			if (buttonGroup == null) {
				buttonGroup = new ButtonGroup();
				view.setProperty(this.group, buttonGroup);
			}
			buttonGroup.add(c);
		}

		// Text expression
		final GUIExpression textExpression = new GUIExpression(view
				.getActionContext().getSubscriberValidator(), view
				.getViewData(), this.text);
		Runnable textTask = new Runnable() {

			@Override
			public void run() {
				String textValue = textExpression.getValue();
				MnemonicInfo info = new MnemonicInfo(textValue);
				info.configureButton(c);
			}
		};
		textExpression.subscribe(view.getActionContext()
				.getSubscriberValidator(), textTask);
		textTask.run();

		// Component -> Model
		c.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (c.isSelected()) {
					setProperty(view, FieldRadio.this.value);
				}
			}
		});

		// Model -> Component
		Runnable task = new Runnable() {
			@Override
			public void run() {
				Object propertyValue = getProperty(view);
				c.setSelected(ObjectUtils.equals(propertyValue,
						FieldRadio.this.value));
			}
		};
		subscribe(view, task);
		task.run();

		// OK
		return new SimpleField<V>(view, this, c);
	}

	/**
	 * Gets the name of the group this radio button belongs to.
	 * 
	 * @return Group name
	 */
	public String getGroup() {
		return this.group;
	}

	/**
	 * Gets the text expression for this radio button.
	 * 
	 * @return Text expression
	 */
	public String getText() {
		return this.text;
	}

	/**
	 * Gets the associated value for this radio button. The radio button will be
	 * selected if the model property is equal to this value. If the radio
	 * button is selected, the model property is update to this value.
	 * 
	 * @return Property value
	 */
	public T getValue() {
		return this.value;
	}

	/**
	 * Sets the name of the group this radio button belongs to.
	 * 
	 * @param group
	 *            Group name
	 */
	public void setGroup(String group) {
		this.group = group;
	}

	/**
	 * Sets the text expression for this radio button.
	 * 
	 * @param text
	 *            Text expression
	 */
	public void setText(String text) {
		this.text = text;
	}

	/**
	 * Sets the associated value for this radio button. The radio button will be
	 * selected if the model property is equal to this value. If the radio
	 * button is selected, the model property is update to this value.
	 * 
	 * @param value
	 *            Property value
	 */
	public void setValue(T value) {
		this.value = value;
	}

}
