package net.sf.doolin.gui.field.list.multi;

import java.awt.SystemColor;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.JTextField;

import net.sf.doolin.gui.action.support.ActionDelegate;
import net.sf.doolin.gui.field.Field;
import net.sf.doolin.gui.field.list.AbstractListPropertyFieldDescriptor;
import net.sf.doolin.gui.swing.LabelInfo;
import net.sf.doolin.gui.swing.LabelInfoProvider;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.EditableValue;

import org.apache.commons.lang.StringUtils;

import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.event.ListEvent;
import ca.odell.glazedlists.event.ListEventListener;

// FIXME Split this field in two: field+popup, editable+list

/**
 * Field descriptor for the selection of several items in a list.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of object in the item list
 */
public class FieldMultiCombo<V, E> extends
		AbstractListPropertyFieldDescriptor<V, E> {

	private static final long serialVersionUID = 1L;

	private int columns = 20;
	private int visibleRows = 8;
	private int timeOut = 2000;
	private boolean canClose = true;

	private ActionDelegate<String> creationDelegate;
	private ActionDelegate<EditableValue<E>> updateDelegate;
	private ActionDelegate<Collection<E>> deleteDelegate;

	@Override
	public Field<V> createField(GUIView<V> view) {
		// Gets the source list
		EventList<E> sourceList = getListModel(view);
		// Gets the selection
		@SuppressWarnings("unchecked")
		final EventList<E> selectionList = (EventList<E>) getProperty(view);

		// Initial text
		String text = getText(selectionList);

		// Text which hosts the selection
		final JTextField rootComponent = new JTextField(text, this.columns);
		rootComponent.setEditable(false);
		rootComponent.setBackground(SystemColor.window);

		// Link the selection list to the text content
		selectionList.addListEventListener(new ListEventListener<E>() {

			@Override
			public void listChanged(ListEvent<E> e) {
				String text = getText(selectionList);
				rootComponent.setText(text);
			}
		});

		// Field instance
		final MultiComboField<V, E> field = new MultiComboField<V, E>(view,
				this, rootComponent, sourceList, selectionList);

		// Mouse adapter
		MultiComboMouseAdapter<V, E> mouseAdapter = new MultiComboMouseAdapter<V, E>(
				field);
		rootComponent.addMouseListener(mouseAdapter);

		// OK
		return field;
	}

	/**
	 * @return the columns
	 */
	public int getColumns() {
		return this.columns;
	}

	public ActionDelegate<String> getCreationDelegate() {
		return this.creationDelegate;
	}

	/**
	 * @return the deleteDelegate
	 */
	public ActionDelegate<Collection<E>> getDeleteDelegate() {
		return this.deleteDelegate;
	}

	/**
	 * Gets a text summary for the selected list of items
	 * 
	 * @param selectionList
	 *            Selected list of items
	 * @return Text summary
	 */
	protected String getText(EventList<E> selectionList) {
		// List of texts
		List<String> textList = new ArrayList<String>();
		// For each selected item
		LabelInfoProvider labelInfoProvider = getLabelInfoProvider();
		for (E item : selectionList) {
			LabelInfo info = labelInfoProvider.getLabelIcon(item);
			String text = info.getText();
			textList.add(text);
		}
		// Join
		return StringUtils.join(textList, File.pathSeparatorChar);
	}

	/**
	 * @return the timeOut
	 */
	public int getTimeOut() {
		return this.timeOut;
	}

	/**
	 * @return the updateDelegate
	 */
	public ActionDelegate<EditableValue<E>> getUpdateDelegate() {
		return this.updateDelegate;
	}

	/**
	 * @return the visibleRows
	 */
	public int getVisibleRows() {
		return this.visibleRows;
	}

	/**
	 * @return the canClose
	 */
	public boolean isCanClose() {
		return this.canClose;
	}

	public boolean isEditable() {
		return this.updateDelegate != null;
	}

	/**
	 * @param canClose
	 *            the canClose to set
	 */
	public void setCanClose(boolean canClose) {
		this.canClose = canClose;
	}

	/**
	 * @param columns
	 *            the columns to set
	 */
	public void setColumns(int columns) {
		this.columns = columns;
	}

	public void setCreationDelegate(ActionDelegate<String> creationDelegate) {
		this.creationDelegate = creationDelegate;
	}

	/**
	 * @param deleteDelegate
	 *            the deleteDelegate to set
	 */
	public void setDeleteDelegate(ActionDelegate<Collection<E>> deleteDelegate) {
		this.deleteDelegate = deleteDelegate;
	}

	/**
	 * @param timeOut
	 *            the timeOut to set
	 */
	public void setTimeOut(int timeOut) {
		this.timeOut = timeOut;
	}

	/**
	 * @param updateDelegate
	 *            the updateDelegate to set
	 */
	public void setUpdateDelegate(
			ActionDelegate<EditableValue<E>> updateDelegate) {
		this.updateDelegate = updateDelegate;
	}

	/**
	 * @param visibleRows
	 *            the visibleRows to set
	 */
	public void setVisibleRows(int visibleRows) {
		this.visibleRows = visibleRows;
	}
}
