package net.sf.doolin.gui.field.table.support;

import java.awt.Container;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.field.table.Column;
import net.sf.doolin.gui.field.table.FieldTable;
import net.sf.doolin.gui.field.table.TableSelectionMode;
import net.sf.doolin.gui.field.table.header.CustomHeaderFactory;
import net.sf.doolin.gui.field.table.renderer.GUITableCellRenderer;
import net.sf.doolin.gui.field.table.style.TableStyle;
import net.sf.doolin.gui.swing.SwingUtils;
import net.sf.doolin.util.Utils;
import net.sf.doolin.util.factory.DataFactory;
import net.sf.doolin.util.unit.ValueUnit;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.FilterList;
import ca.odell.glazedlists.SortedList;
import ca.odell.glazedlists.gui.AbstractTableComparatorChooser;
import ca.odell.glazedlists.matchers.MatcherEditor;
import ca.odell.glazedlists.swing.EventTableModel;
import ca.odell.glazedlists.swing.TableComparatorChooser;
import ca.odell.glazedlists.util.concurrent.Lock;

/**
 * {@link JTable} implementation for the Doolin GUI framework, for integration
 * as a {@link net.sf.doolin.gui.field.Field}.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of item in the table
 */
public class GUITable<V, E> extends JTable {

	private static final Logger log = LoggerFactory.getLogger(GUITable.class);

	private static final long serialVersionUID = 1L;

	private final GUITableField<V, E> tableField;
	private GUITableFormat<V, E> tableFormat;

	private EventTableModel<E> tableModel;
	private EventList<E> tableModelList;

	private SortedList<E> sortedList;

	private TableComparatorChooser<E> tableComparatorChooser;

	/**
	 * Instantiates a new GUI table.
	 * 
	 * @param tableField
	 *            Associated table field
	 */
	public GUITable(GUITableField<V, E> tableField) {
		this.tableField = tableField;
		FieldTable<V, E> table = tableField.getTableDescriptor();
		// Selection mode
		final TableSelectionMode selectionMode = table.getSelectionMode();
		setSelectionMode(selectionMode.getSelectionMode());
		// Header
		JTableHeader header = table.getHeaderFactory().createHeader(this);
		setTableHeader(header);
		// Listens for re-initialisation of the list
		final ActionContext actionContext = tableField.getActionContext();
		Runnable initTask = new Runnable() {

			@Override
			public void run() {
				reinit();
			}
		};
		PropertyChangeSupport.subscribe(actionContext.getSubscriberValidator(),
				actionContext.getData(), tableField.getTableDescriptor()
						.getItemListFactory().getPropertyPath(), initTask);
		// Initialization
		initTask.run();
		// Mouse listener
		addMouseListener(new GUITableMouseListener<V, E>(this));
		// Selection listener
		final String selectionPropertyPath = table.getSelectionPropertyPath();
		if (StringUtils.isNotBlank(selectionPropertyPath)) {
			// Selection lock
			final AtomicBoolean onSelection = new AtomicBoolean(false);
			// Table -> Model
			getSelectionModel().addListSelectionListener(
					new ListSelectionListener() {

						@Override
						public void valueChanged(ListSelectionEvent e) {
							if (!e.getValueIsAdjusting() && !onSelection.get()) {
								onSelection.set(true);
								try {
									Object viewData = GUITable.this.tableField
											.getActionContext().getData();
									if (selectionMode.isMultiple()) {
										List<E> items = new ArrayList<E>();
										int[] rows = getSelectedRows();
										for (int row : rows) {
											E item = GUITable.this.tableModel
													.getElementAt(row);
											items.add(item);
										}
										Utils.setProperty(viewData,
												selectionPropertyPath, items);
									} else {
										int selectedRow = getSelectedRow();
										if (selectedRow < 0) {
											Utils
													.setProperty(
															viewData,
															selectionPropertyPath,
															null);
										} else if (selectedRow < GUITable.this.tableModel
												.getRowCount()) {
											E item = GUITable.this.tableModel
													.getElementAt(selectedRow);
											Utils
													.setProperty(
															viewData,
															selectionPropertyPath,
															item);
										}
									}
								} finally {
									onSelection.set(false);
								}
							}
						}

					});
			// Model -> Selection
			PropertyChangeSupport.subscribe(actionContext
					.getSubscriberValidator(), actionContext.getData(),
					selectionPropertyPath, new Runnable() {

						@Override
						public void run() {
							if (!onSelection.get()) {
								onSelection.set(true);
								try {
									if (selectionMode.isMultiple()) {
										// Gets the selection
										@SuppressWarnings("unchecked")
										List<E> itemList = (List<E>) Utils
												.getProperty(actionContext
														.getData(),
														selectionPropertyPath);
										// Changes the selection
										if (itemList != null) {
											final Lock lock = GUITable.this.tableModelList
													.getReadWriteLock()
													.readLock();
											lock.lock();
											try {
												getSelectionModel()
														.clearSelection();
												for (E item : itemList) {
													int index = GUITable.this.tableModelList
															.indexOf(item);
													getSelectionModel()
															.addSelectionInterval(
																	index,
																	index);
												}
											} finally {
												lock.unlock();
											}
										} else {
											setRowSelectionInterval(-1, -1);
										}
									} else {
										// Gets the selection
										@SuppressWarnings("unchecked")
										E item = (E) Utils.getProperty(
												actionContext.getData(),
												selectionPropertyPath);
										// Changes the selection
										if (item != null) {
											int index = GUITable.this.tableModelList
													.indexOf(item);
											log
													.debug(String
															.format(
																	"Index of selection %s id %s",
																	item, index));
											if (index >= 0) {
												setRowSelectionInterval(index,
														index);
											}
										} else {
											clearSelection();
										}
									}
								} finally {
									onSelection.set(false);
								}
							}
						}
					});
		}
	}

	@Override
	protected void configureEnclosingScrollPane() {
		FieldTable<V, E> tableDescriptor = this.tableField.getTableDescriptor();
		// Customises the scroll pane header
		CustomHeaderFactory<V, E> customHeaderFactory = tableDescriptor
				.getCustomHeaderFactory();
		if (customHeaderFactory == null) {
			super.configureEnclosingScrollPane();
		} else {
			JComponent headerComponent = customHeaderFactory
					.createHeaderComponent(this);
			if (headerComponent != null) {
				/*
				 * Thanks to tip described at:
				 * http://forums.sun.com/thread.jspa?threadID=693708&tstart=60
				 */
				Container p = getParent();
				if (p instanceof JViewport) {
					Container gp = p.getParent();
					if (gp instanceof JScrollPane) {
						JScrollPane scrollPane = (JScrollPane) gp;
						// Make certain we are the viewPort's view and not, for
						// example, the rowHeaderView of the scrollPane -
						// an implementor of fixed columns might do this.
						JViewport viewport = scrollPane.getViewport();
						if (viewport == null || viewport.getView() != this) {
							return;
						}
						scrollPane.setColumnHeaderView(headerComponent);
					} else {
						log
								.warn("Cannot set custom header when the parent is not a scroll pane");
					}
				}
			} else {
				super.configureEnclosingScrollPane();
			}
		}
	}

	/**
	 * Gets the table model that is based on {@link EventList}.
	 * 
	 * @return Table model
	 */
	@SuppressWarnings("unchecked")
	public EventTableModel<E> getEventTableModel() {
		return (EventTableModel<E>) getModel();
	}

	/**
	 * Gets an item from the table
	 * 
	 * @param row
	 *            Row index
	 * @return Item
	 */
	public E getItemAt(int row) {
		return getEventTableModel().getElementAt(row);
	}

	/**
	 * Gets the selected item. If several items are selected, returns the first
	 * one.
	 * 
	 * @return Selected item or <code>null</code> if the selection is empty
	 */
	public E getSelectedItem() {
		int row = getSelectedRow();
		if (row < 0) {
			return null;
		} else if (row < this.tableModel.getRowCount()) {
			return getItemAt(row);
		} else {
			return null;
		}
	}

	/**
	 * Gets the list of selected items.
	 * 
	 * @return Selected item or <code>null</code> if the selection is empty
	 */
	public List<E> getSelectedItems() {
		List<E> list = new ArrayList<E>();
		int[] rows = getSelectedRows();
		for (int i = 0; i < rows.length; i++) {
			int row = rows[i];
			E item = getItemAt(row);
			list.add(item);
		}
		return list;
	}

	/**
	 * Gets the table field.
	 * 
	 * @return the table field
	 */
	public GUITableField<V, E> getTableField() {
		return this.tableField;
	}

	/**
	 * Gets the style for this table
	 * 
	 * @return Style
	 * @see GUITableField#getTableDescriptor()
	 * @see FieldTable#getTableStyle()
	 */
	public TableStyle getTableStyle() {
		return this.tableField.getTableDescriptor().getTableStyle();
	}

	/**
	 * Initialises the columns.
	 */
	protected void initColumns() {
		// Column model
		TableColumnModel columnModel = getColumnModel();
		// For each column
		int colCount = getColumnCount();
		for (int colIndex = 0; colIndex < colCount; colIndex++) {
			// Column
			TableColumn tableColumn = columnModel.getColumn(colIndex);
			// Index in the model
			int modelIndex = tableColumn.getModelIndex();
			// Column definition
			Column column = this.tableFormat.getActualColumn(modelIndex);
			setupColumn(colIndex, tableColumn, column, true);
		}
	}

	/**
	 * Initialises the model.
	 */
	protected void initModel() {
		// Gets the item list
		EventList<E> sourceList = this.tableField.getList();
		// Sorted list
		this.sortedList = new SortedList<E>(sourceList, null);
		EventList<E> modelList = this.sortedList;
		// Filtered list
		DataFactory<MatcherEditor<E>> matcherEditorFactory = getTableField()
				.getTableDescriptor().getMatcherEditorFactory();
		if (matcherEditorFactory != null) {
			MatcherEditor<E> matcherEditor = matcherEditorFactory
					.create(this.tableField.getActionContext().getData());
			if (matcherEditor != null) {
				FilterList<E> filterList = new FilterList<E>(this.sortedList,
						matcherEditor);
				modelList = filterList;
			}
		}
		// Creates the format
		this.tableFormat = new GUITableFormat<V, E>(this.tableField);
		// Creates the model
		this.tableModelList = modelList;
		this.tableModel = new EventTableModel<E>(modelList, this.tableFormat) {

			private static final long serialVersionUID = 1L;

			@Override
			public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
				// Super
				super.setValueAt(aValue, rowIndex, columnIndex);
				// Column
				Column column = GUITable.this.tableFormat
						.getActualColumn(columnIndex);
				// Update mode
				switch (column.getColumnUpdatePolicy()) {
				case TABLE:
					fireTableDataChanged();
					break;
				case ROW:
					fireTableRowsUpdated(rowIndex, rowIndex);
					break;
				case COLUMN:
					int size = getRowCount();
					for (int row = 0; row < size; row++) {
						fireTableCellUpdated(row, columnIndex);
					}
					break;
				default:
					fireTableCellUpdated(rowIndex, columnIndex);
					break;
				}
			}

		};
		// Sets the model
		setModel(this.tableModel);
	}

	/**
	 * Initializes sorting capabilities
	 */
	protected void initSorting() {
		if (getTableHeader() != null) {
			if (this.tableComparatorChooser != null) {
				this.tableComparatorChooser.dispose();
			}
			this.tableComparatorChooser = TableComparatorChooser.install(this,
					this.sortedList,
					AbstractTableComparatorChooser.MULTIPLE_COLUMN_MOUSE);
		}
	}

	/**
	 * Reinitialises the table: model, columns and sorting.
	 */
	public void reinit() {
		initModel();
		initColumns();
		initSorting();
	}

	/**
	 * Set-up a column
	 * 
	 * @param colIndex
	 *            Column index
	 * @param tableColumn
	 *            Associated column model
	 * @param column
	 *            Column descriptor
	 * @param setupWidth
	 *            <code>true</code> if the width must be set
	 */
	protected void setupColumn(int colIndex, TableColumn tableColumn,
			Column column, boolean setupWidth) {
		// Renderer
		setupRenderer(colIndex, tableColumn, column);
		// Editor
		setupEditor(colIndex, tableColumn, column);
		// Size
		ValueUnit width = column.getWidth();
		if (width != null && setupWidth) {
			SwingUtils.setTableColumnWidth(this, colIndex, width, column
					.isResizeable());
		}
	}

	/**
	 * Setup the editor for one column
	 * 
	 * @param colIndex
	 *            Column index
	 * @param tableColumn
	 *            Column in the JTable column model
	 * @param column
	 *            Column definition
	 */
	protected void setupEditor(int colIndex, TableColumn tableColumn,
			Column column) {
		TableCellEditor cellEditor = column.getEditor();
		if (cellEditor != null) {
			tableColumn.setCellEditor(cellEditor);
		}
	}

	/**
	 * Setup the renderer for one column
	 * 
	 * @param colIndex
	 *            Column index
	 * @param tableColumn
	 *            Column in the JTable column model
	 * @param column
	 *            Column definition
	 */
	protected void setupRenderer(int colIndex, TableColumn tableColumn,
			Column column) {
		// Cell renderer
		TableCellRenderer cellRenderer = column.getRenderer();
		if (cellRenderer == null) {
			// Gets the default
			cellRenderer = getDefaultRenderer(column.getColumnClass());
		}
		// Wraps the renderer
		cellRenderer = new GUITableCellRenderer<E>(cellRenderer);
		// OK
		tableColumn.setCellRenderer(cellRenderer);
	}
}
