package net.sf.doolin.gui.field.table.support;

import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.EventListener;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import net.sf.doolin.bus.Bus;
import net.sf.doolin.bus.SubscriberValidator;
import net.sf.doolin.bus.support.AndSubscriberValidator;
import net.sf.doolin.bus.support.PropertyChangeMessage;
import net.sf.doolin.bus.support.PropertyChangeSubscriber;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.action.path.ActionBarContext;
import net.sf.doolin.gui.action.path.list.ActionPathList;
import net.sf.doolin.gui.action.path.toolbar.ToolBar;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.field.support.AbstractField;
import net.sf.doolin.gui.field.table.FieldTable;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.gui.swing.DockLayout;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.support.GUIViewAdapter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.ObservableElementList;

/**
 * Field for a table.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of item in the table
 */
public class GUITableField<V, E> extends AbstractField<V> {

	private static final Logger log = LoggerFactory
			.getLogger(GUITableField.class);

	private final FieldTable<V, E> tableDescriptor;
	private final JScrollPane scrollPane;
	private final JPanel tableContainer;
	private final GUITable<V, E> table;

	/**
	 * Constructor.
	 * 
	 * @param view
	 *            Hosting view
	 * @param fieldDescriptor
	 *            Table descriptor
	 */
	public GUITableField(GUIView<V> view, FieldTable<V, E> fieldDescriptor) {
		super(view, fieldDescriptor);
		this.tableDescriptor = fieldDescriptor;
		// Creates the container
		this.tableContainer = new JPanel(new DockLayout());
		// Creates the table
		this.table = new GUITable<V, E>(this);
		// Table context
		setActionContext(new TableActionContext<V, E>(this));
		// Creates the scroll pane
		this.scrollPane = new JScrollPane(this.table);
		this.tableContainer.add(this.scrollPane, BorderLayout.CENTER);
		// Action context
		ActionContext actionContext = getActionContext();
		// Table Tool bar
		ActionPathList toolBar = this.tableDescriptor.getTableToolBar();
		if (toolBar != null) {
			// Creates the tool bar
			JToolBar jtoolBar = this.tableDescriptor
					.getActionFactory()
					.createToolBar(new ActionBarContext(toolBar, actionContext));
			jtoolBar.setFloatable(false);
			// Adds the tool bar
			this.tableContainer.add(jtoolBar, BorderLayout.NORTH);
		}
		// View Tool bar
		ActionPathList viewToolBar = this.tableDescriptor.getViewToolBar();
		if (viewToolBar != null) {
			// Adds the tool bar
			view.installToolBar(viewToolBar, actionContext);
		}
		// Window Tool bar
		ToolBar windowToolBar = this.tableDescriptor.getWindowToolBar();
		if (windowToolBar != null) {
			// Registers for installation
			view.addViewListener(new GUIViewAdapter<V>() {
				@Override
				public void onViewClosed(GUIView<V> view) {
					view.getParentWindow().uninstallToolBar(
							GUITableField.this.tableDescriptor
									.getWindowToolBar().getName());
				}

				@Override
				public void onViewInit(GUIView<V> view) {
					// Adds the tool bar
					view.getParentWindow().installToolBar(
							GUITableField.this.tableDescriptor
									.getWindowToolBar(), getActionContext());
				}
			});
		}
		// Key bindings
		List<GUIAction> tableActions = this.tableDescriptor.getTableActions();
		if (tableActions != null) {
			for (GUIAction action : tableActions) {
				// Action shortcut
				GUIExpression shortcutExpression = action
						.getShortcutExpression(actionContext);
				if (shortcutExpression != null) {
					String shortcut = shortcutExpression.getValue();
					// Binds the action
					String actionID = action.getId();
					this.table.getInputMap().put(
							KeyStroke.getKeyStroke(shortcut), actionID);
					this.table.getActionMap().put(
							actionID,
							fieldDescriptor.getActionFactory().createAction(
									actionContext, action, IconSize.SMALL));
				}
			}
		}
	}

	@Override
	public JComponent getCoreComponent() {
		return this.table;
	}

	@Override
	public JComponent getFieldComponent() {
		return this.tableContainer;
	}

	/**
	 * Gets the event list which is used for the table model.
	 * <p>
	 * This list is based on the event list that is returned from the model (see
	 * {@link #getListModel()}) but is wrapped by an
	 * {@link ObservableElementList} if the <code>itemClass</code> property of
	 * the table descriptor is set. This allows the rows to be automatically
	 * updated if the item in the row is updated.
	 * 
	 * @return Event list used by the table model
	 */
	public EventList<E> getList() {
		EventList<E> eventList = getListModel();
		// Connector ?
		if (this.tableDescriptor.isItemChanges()) {

			log.debug("Getting the list for the table field");

			final Set<E> registeredElements = new HashSet<E>();
			final SubscriberValidator tableSubscriptionValidator = getActionContext()
					.getSubscriberValidator();

			ObservableElementList.Connector<E> connector = new ObservableElementList.Connector<E>() {

				private ObservableElementList<E> list;

				private PropertyChangeListener listener = new PropertyChangeListener() {

					@SuppressWarnings("unchecked")
					@Override
					public void propertyChange(PropertyChangeEvent evt) {
						list.elementChanged((E) evt.getSource());
					}
				};

				@Override
				public EventListener installListener(final E element) {
					log.debug(String.format("Installing listener for %s",
							element));
					registeredElements.add(element);
					AndSubscriberValidator combinedSubscriberValidator = new AndSubscriberValidator(
							tableSubscriptionValidator,
							new SubscriberValidator() {

								@Override
								public String getValidatorDescription() {
									return String
											.format("Element %s contained by the table",
													element);
								}

								@Override
								public boolean isValid() {
									return registeredElements.contains(element);
								}
							});
					net.sf.doolin.bus.support.PropertyChangeListener propertyChangeListener = new net.sf.doolin.bus.support.PropertyChangeListener() {

						@Override
						public void onChange(PropertyChangeMessage message) {
							String propertyName = message.getPropertyName();
							Object propertyValue = message.getPropertyValue();
							log.debug(
									"{} element {} property has changed to {}",
									new Object[] { element, propertyName,
											propertyValue });
							listener.propertyChange(new PropertyChangeEvent(
									element, propertyName, null, propertyValue));
						}
					};
					Bus.get()
							.subscribe(
									new PropertyChangeSubscriber(
											String.format(
													"Connects between table element %s and the table model",
													element),
											combinedSubscriberValidator,
											element, null,
											propertyChangeListener));
					return this.listener;
				}

				@SuppressWarnings("unchecked")
				@Override
				public void setObservableElementList(
						ObservableElementList<? extends E> list) {
					this.list = (ObservableElementList<E>) list;
				}

				@Override
				public void uninstallListener(E element, EventListener listener) {
					log.debug(String.format("Uninstalling listener for %s",
							element));
					registeredElements.remove(element);
				}
			};
			eventList = new ObservableElementList<E>(eventList, connector);
		}
		// OK
		return eventList;
	}

	/**
	 * Returns the value model that contains the list of items.
	 * 
	 * @return Value model for the list of items
	 */
	public EventList<E> getListModel() {
		// Gets the model
		V model = getView().getViewData();
		// Gets the list model
		EventList<E> list = this.tableDescriptor.getItemListFactory()
				.createEventList(getActionContext().getSubscriberValidator(),
						model);
		// Do not accept null
		if (list == null) {
			list = new BasicEventList<E>();
		}
		// OK
		return list;
	}

	/**
	 * Returns the associated table component.
	 * 
	 * @return Table component
	 */
	public GUITable<V, E> getTable() {
		return this.table;
	}

	/**
	 * Gets the associated table descriptor.
	 * 
	 * @return Table descriptor
	 */
	public FieldTable<V, E> getTableDescriptor() {
		return this.tableDescriptor;
	}

}
