package net.sf.doolin.gui.field.table.support;

import java.util.Comparator;
import java.util.List;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.field.table.Column;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ca.odell.glazedlists.gui.AdvancedTableFormat;
import ca.odell.glazedlists.gui.WritableTableFormat;

/**
 * Format specification for the table. This implementation delegates most of its
 * calls to the {@link Column} class.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of item in the table
 */
public class GUITableFormat<V, E> implements AdvancedTableFormat<E>,
		WritableTableFormat<E> {

	private final static Logger log = LoggerFactory.getLogger(GUITableFormat.class);

	private final GUITableField<V, E> tableField;

	private List<Column> availableColumns;
	private List<Column> actualColumns;

	/**
	 * Instantiates a new GUI table format.
	 * 
	 * @param tableField
	 *            the table field
	 */
	public GUITableFormat(GUITableField<V, E> tableField) {
		this.tableField = tableField;
		init();
	}

	/**
	 * Gets the actual column.
	 * 
	 * @param modelIndex
	 *            the model index
	 * 
	 * @return the actual column
	 */
	public Column getActualColumn(int modelIndex) {
		return this.actualColumns.get(modelIndex);
	}

	/**
	 * @see Column#getColumnClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class getColumnClass(int index) {
		Column column = this.actualColumns.get(index);
		return column.getColumnClass();
	}

	/**
	 * @see Column#getComparator()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Comparator getColumnComparator(int index) {
		Column column = this.actualColumns.get(index);
		return column.getComparator();
	}

	/**
	 * Number of columns.
	 */
	@Override
	public int getColumnCount() {
		return this.actualColumns.size();
	}

	/**
	 * @see Column#getTitleExpression()
	 */
	@Override
	public String getColumnName(final int index) {
		Column column = this.actualColumns.get(index);
		String title = column.getTitleExpression();
		final GUIExpression titleExpression = new GUIExpression(this.tableField
				.getActionContext().getSubscriberValidator(), this.tableField
				.getView().getViewData(), title);
		// Title changes
		titleExpression.subscribe(this.tableField.getActionContext()
				.getSubscriberValidator(), new Runnable() {

			@Override
			public void run() {
				GUITableFormat.this.tableField.getTable().getColumnModel()
						.getColumn(index).setHeaderValue(
								titleExpression.getValue());
			}
		});
		// OK
		return titleExpression.getValue();
	}

	/**
	 * @see Column#getValueFactory()
	 * @see net.sf.doolin.gui.field.table.ColumnValueFactory#getValue(Object)
	 */
	@Override
	public Object getColumnValue(Object item, int index) {
		Column column = this.actualColumns.get(index);
		return column.getValueFactory().getValue(item);
	}

	/**
	 * @see Column#isEditable(Object)
	 */
	@Override
	public boolean isEditable(Object baseObject, int columnIndex) {
		Column column = getActualColumn(columnIndex);
		return column.isEditable(baseObject);
	}

	/**
	 * Updates the column value and executes any action if it is defined.
	 * 
	 * @see Column#getValueFactory()
	 * @see net.sf.doolin.gui.field.table.ColumnValueFactory#setValue(Object,
	 *      Object)
	 * @see Column#getOnEditAction()
	 */
	@Override
	public E setColumnValue(E baseObject, Object editedValue, int columnIndex) {
		Column column = getActualColumn(columnIndex);
		if (column.isEditable(baseObject)) {
			// Edits the value
			column.getValueFactory().setValue(baseObject, editedValue);
			// Calls any onEdit action
			GUIAction onEditAction = column.getOnEditAction();
			if (onEditAction != null) {
				// Action context
				ActionContext actionContext = this.tableField
						.getActionContext();
				// Execution
				onEditAction.execute(actionContext);
			}
			// OK
			return baseObject;
		} else {
			log
					.warn("Tried to edit a non-editable column: "
							+ column.getName());
			return null;
		}
	}

	/**
	 * Initialisation.
	 */
	protected void init() {
		// Loads the columns
		this.availableColumns = this.tableField.getTableDescriptor()
				.getColumnGenerator().generateColumns(this.tableField);
		// Filters the columns
		this.actualColumns = Column.filter(this.availableColumns);
	}

}
