package net.sf.doolin.gui.field.tree.support;

import java.util.Enumeration;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.field.tree.FieldTree;
import net.sf.doolin.gui.field.tree.FieldTreeSelectionPropertyMode;
import net.sf.doolin.gui.field.tree.model.GUITreeModel;
import net.sf.doolin.gui.field.tree.model.GUITreeNode;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;

public class GUITree<V> extends JTree {

	private final GUITreeField<V> treeField;

	public GUITree(GUITreeField<V> treeField) {
		this.treeField = treeField;
		FieldTree<V> treeDescriptor = this.treeField.getTreeDescriptor();
		// Visible root
		setRootVisible(treeDescriptor.isRootVisible());
		// Selection mode
		setupSelection();
		// Cell renderer
		setCellRenderer(new GUITreeRenderer());
		// TODO Listener for clicks
		// Model
		Object root = treeField.getProperty();
		GUITreeModel model = new GUITreeModel(treeField.getActionContext()
				.getSubscriberValidator(), root, treeDescriptor.getNodes());
		setModel(model);
	}

	private TreePath findPath(Object value, boolean useUserObject) {
		TreeNode rootNode = (TreeNode) getModel().getRoot();
		TreePath rootPath = new TreePath(rootNode);
		return findPath(rootPath, value, useUserObject);
	}

	private TreePath findPath(TreePath parentPath, Object value,
			boolean useUserObject) {
		if (sameNode(parentPath, value, useUserObject)) {
			return parentPath;
		} else {
			GUITreeNode parentNode = (GUITreeNode) parentPath
					.getLastPathComponent();
			if (parentNode.getChildCount() > 0) {
				@SuppressWarnings("unchecked")
				Enumeration<GUITreeNode> e = parentNode.children();
				while (e.hasMoreElements()) {
					GUITreeNode childNode = e.nextElement();
					TreePath childPath = parentPath
							.pathByAddingChild(childNode);
					TreePath result = findPath(childPath, value, useUserObject);
					if (result != null) {
						return result;
					}
				}
			}
			// Not found
			return null;
		}
	}

	public Object getSelection() {
		FieldTree<V> treeDescriptor = this.treeField.getTreeDescriptor();
		TreePath path = getSelectionPath();
		if (path == null) {
			return null;
		} else {
			GUITreeNode node = (GUITreeNode) path.getLastPathComponent();
			FieldTreeSelectionPropertyMode selectionPropertyMode = treeDescriptor
					.getSelectionPropertyMode();
			switch (selectionPropertyMode) {
			case PATH:
				return path;
			case NODE:
				return node;
			case OBJECT:
				return node.getUserObject();
			default:
				throw new IllegalStateException("Unknown tree selection mode: "
						+ selectionPropertyMode);
			}
		}
	}

	private boolean sameNode(TreePath parentPath, Object value,
			boolean useUserObject) {
		GUITreeNode node = (GUITreeNode) parentPath.getLastPathComponent();
		if (useUserObject) {
			return (node.getUserObject() == value);
		} else {
			return (node == value);
		}
	}

	protected void setSelectionProperty(Object value) {
		Object treeData = this.treeField.getActionContext().getData();
		final FieldTree<V> treeDescriptor = this.treeField.getTreeDescriptor();
		String selectionPropertyPath = treeDescriptor
				.getSelectionPropertyPath();
		if (StringUtils.isNotBlank(selectionPropertyPath)) {
			Utils.setProperty(treeData, selectionPropertyPath, value);
		}
	}

	private void setupSelection() {
		FieldTree<V> treeDescriptor = this.treeField.getTreeDescriptor();
		// Single selection
		getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		// Is selection enabled?
		final String selectionPropertyPath = treeDescriptor
				.getSelectionPropertyPath();
		if (StringUtils.isNotBlank(selectionPropertyPath)) {
			// Lock
			final AtomicBoolean onUpdate = new AtomicBoolean();
			// GUI --> Model
			getSelectionModel().addTreeSelectionListener(
					new TreeSelectionListener() {

						@Override
						public void valueChanged(TreeSelectionEvent e) {
							if (!onUpdate.get()) {
								setSelectionProperty(getSelection());
							}
						}
					});
			// Model --> GUI
			final V model = this.treeField.getView().getViewData();
			PropertyChangeSupport.subscribe(this.treeField.getActionContext()
					.getSubscriberValidator(), model, selectionPropertyPath,
					new Runnable() {

						@Override
						public void run() {
							onUpdate.set(true);
							try {
								// Gets the new value
								Object newValue = Utils.getProperty(model,
										selectionPropertyPath);
								// No selection ?
								if (newValue == null) {
									clearSelection();
								} else {
									// Mapping to the path
									TreePath selectionPath = toPath(newValue);
									// Selects the path
									if (selectionPath != null) {
										setSelectionPath(selectionPath);
									} else {
										clearSelection();
									}
								}
								// Missing code for Type1271842262949.run
							} finally {
								onUpdate.set(false);
							}
						}
					});
		}
	}

	private TreePath toPath(Object value) {
		FieldTree<V> treeDescriptor = this.treeField.getTreeDescriptor();
		FieldTreeSelectionPropertyMode mode = treeDescriptor
				.getSelectionPropertyMode();
		switch (mode) {
		case NODE:
			return findPath(value, false);
		case OBJECT:
			return findPath(value, true);
		case PATH:
			return (TreePath) value;
		default:
			throw new IllegalStateException("Unknown tree selection mode: "
					+ mode);
		}
	}
}
