package net.sf.doolin.gui.monitor;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.UndeclaredThrowableException;
import java.util.concurrent.ExecutionException;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.UIManager;

/**
 * Monitor that displays a progress dialog for a {@link Task}.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <T>
 *            the result type returned by the {@link Task} {@code
 *            doInBackground} and {@code get} methods
 * @param <V>
 *            the type used for carrying out intermediate results by this
 *            {@link Task} {@code publish} and {@code process} methods
 */
public class TaskMonitor<T, V> implements TaskListener<T, V>,
		PropertyChangeListener {

	private final Task<T, V> task;
	private final JComponent component;
	private final String title;
	private final Object messageList;

	private int maximum;

	private boolean cancellable = true;
	private String initialNote;

	private String cancelOption;

	private JLabel labelNote;
	private JProgressBar progressBar;
	private JOptionPane optionPane;
	private JDialog dialog;

	private boolean cancelled;

	/**
	 * Constructor
	 * 
	 * @param task
	 *            Task to run
	 * @param component
	 *            Parent component for the progress dialog
	 * @param title
	 *            Title for the dialog
	 * @param messageList
	 *            List of message, as specified in {@link JOptionPane}
	 * @param maximum
	 *            Maximum of steps for the progress bar. If set to 0, the
	 *            progress bar is set
	 *            {@linkplain JProgressBar#setIndeterminate(boolean)
	 *            indeterminate}.
	 */
	public TaskMonitor(Task<T, V> task, JComponent component, String title,
			Object messageList, int maximum) {
		this.task = task;
		this.component = component;
		this.title = title;
		this.messageList = messageList;
		this.maximum = maximum;
	}

	/**
	 * Returns the cancellation state
	 * 
	 * @return <code>true</code> if the task has been cancelled.
	 */
	public boolean isCancelled() {
		return this.cancelled;
	}

	@Override
	public void onNoteChanged(Task<T, V> task, final String note) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				TaskMonitor.this.labelNote.setText(note);
			}

		});
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		if (this.task == evt.getSource()) {
			String propertyName = evt.getPropertyName();
			if ("progress".equals(propertyName)) {
				int progress = (Integer) evt.getNewValue();
				if (this.progressBar != null) {
					this.progressBar.setValue(progress);
				}
			} else if ("state".equals(propertyName)) {
				SwingWorker.StateValue state = (SwingWorker.StateValue) evt
						.getNewValue();
				switch (state) {
				case DONE:
					this.cancelled = this.task.isCancelled();
					if (this.dialog != null) {
						this.dialog.dispose();
					}
					break;
				}
			}
		}
	}

	/**
	 * Sets if the task can be cancelled or not
	 * 
	 * @param cancellable
	 *            Cancellable state
	 */
	public void setCancellable(boolean cancellable) {
		this.cancellable = cancellable;
	}

	/**
	 * Sets the initial note message for the progress dialog
	 * 
	 * @param note
	 *            Note message
	 */
	public void setInitialNote(String note) {
		this.initialNote = note;
	}

	/**
	 * Updates the maximum number of steps of the progress bar
	 * 
	 * @param max
	 *            Number of steps
	 */
	public void setMaximum(int max) {
		this.maximum = max;
		if (this.progressBar != null) {
			SwingUtilities.invokeLater(new Runnable() {

				public void run() {
					TaskMonitor.this.progressBar
							.setMaximum(TaskMonitor.this.maximum);
					TaskMonitor.this.progressBar
							.setIndeterminate(0 == TaskMonitor.this.maximum);
				}

			});
		}
	}

	/**
	 * Starts the task
	 * 
	 * @return Result of the task
	 * @throws TaskException
	 *             If the task throws an exception during its execution
	 */
	public T start() throws TaskException {
		// Creates the dialog
		this.dialog = createDialog();
		this.dialog.pack();
		// Setup the task for monitoring
		this.task.setListener(this);
		this.task.addPropertyChangeListener(this);
		// Starts the tread
		this.task.execute();
		// Starts the monitor
		this.dialog.setVisible(true);
		// Canceled ?
		if (this.optionPane.getValue() != null
				&& this.optionPane.getValue().equals(this.cancelOption)) {
			this.cancelled = true;
			this.task.cancel(true);
			return null;
		} else {
			try {
				try {
					return this.task.get();
				} catch (ExecutionException exex) {
					throw exex.getCause();
				}
			} catch (UndeclaredThrowableException utex) {
				Throwable undeclaredThrowable = utex.getUndeclaredThrowable();
				if (undeclaredThrowable instanceof Exception) {
					throw new TaskException((Exception) undeclaredThrowable);
				} else {
					throw new TaskException(new RuntimeException(
							undeclaredThrowable));
				}
			} catch (Exception e) {
				throw new TaskException(e);
			} catch (Throwable e) {
				throw new TaskException(new RuntimeException(e));
			}
		}
	}

	/**
	 * Creates the dialog
	 * 
	 * @return Created dialog
	 */
	protected JDialog createDialog() {
		// Cancel option
		this.cancelOption = UIManager.getString("OptionPane.cancelButtonText");
		// Label for the note
		this.labelNote = new JLabel(this.initialNote != null ? this.initialNote
				: " ");
		this.labelNote.setFont(this.labelNote.getFont().deriveFont(
				(float) (this.labelNote.getFont().getSize() - 1)));
		// Progress bar
		this.progressBar = new JProgressBar();
		this.progressBar.setMaximum(this.maximum);
		this.progressBar.setIndeterminate(this.maximum == 0);
		if (this.cancellable) {
			this.optionPane = new JOptionPane(new Object[] { this.messageList,
					this.labelNote, this.progressBar },
					JOptionPane.INFORMATION_MESSAGE,
					JOptionPane.DEFAULT_OPTION, null,
					new Object[] { this.cancelOption }, null);
		} else {
			this.optionPane = new JOptionPane(new Object[] { this.messageList,
					this.labelNote, this.progressBar },
					JOptionPane.INFORMATION_MESSAGE);
		}
		// Displays the dialog
		this.dialog = this.optionPane.createDialog(this.component, this.title);
		// OK
		return this.dialog;
	}
}
