package net.sf.doolin.gui.service.icon;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Icon;

import net.sf.doolin.gui.service.IconService;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.util.URLUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Icon manager that loads icon from a map of icon definitions.
 * 
 * @author Damien Coraboeuf
 */
public abstract class AbstractIconService implements IconService {

	/**
	 * Associations between icon sizes and actual icons.
	 * 
	 * @author Damien Coraboeuf
	 */
	protected static class IconItem {

		/**
		 * Associations between icon sizes and actual icons.
		 */
		protected Map<IconSize, Icon> iconObjects = new HashMap<IconSize, Icon>();

	}

	/**
	 * Logger
	 */
	private final static Logger log = LoggerFactory.getLogger(AbstractIconService.class);

	private Map<String, IconDefinition> iconDefinitions = new HashMap<String, IconDefinition>();

	private Map<String, IconItem> iconItems;

	/**
	 * Constructor.
	 * 
	 * Registers the "icon:" protocol.
	 * 
	 * @see URLUtils#registerURLStreamHandler(String, java.net.URLStreamHandler)
	 */
	public AbstractIconService() {
		// Icon protocol
		URLUtils.registerURLStreamHandler(IconURLStreamHandler.PROTOCOL_ICON,
				new IconURLStreamHandler(this));
	}

	/**
	 * Looks for the icon ata given size. If the icon is not found, a smaller
	 * icon is looked for.
	 */
	public Icon getIcon(String id, IconSize size) {
		init();
		IconItem iconItem = this.iconItems.get(id);
		if (iconItem == null) {
			log.warn("Cannot find any icon for id = " + id);
			return null;
		} else {
			Icon iconObject = null;
			while (iconObject == null) {
				iconObject = iconItem.iconObjects.get(size);
				if (iconObject != null) {
					return iconObject;
				} else {
					// Tries a smaller size
					size = size.getSmallerSize();
					// No smaller size
					if (size == null) {
						break;
					}
				}
			}
			// Icon not found
			log.warn("Cannot find any icon for id = " + id + " and size = "
					+ size);
			return null;
		}
	}

	/**
	 * Gets the list of icon definitions
	 * 
	 * @return List of icon definitions
	 */
	public Map<String, IconDefinition> getIconDefinitions() {
		return this.iconDefinitions;
	}

	public URL getIconURL(String iconId, IconSize size) {
		init();
		IconDefinition iconDefinition = this.iconDefinitions.get(iconId);
		if (iconDefinition != null) {
			String path = iconDefinition.getPaths().get(size);
			if (path != null) {
				return toURL(path);
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	/**
	 * Initializes the manager. This is a lazy initialization.
	 * 
	 * @see #load()
	 */
	protected void init() {
		if (this.iconItems == null) {
			load();
		}
	}

	/**
	 * This method actually initializes the icon manager.
	 */
	protected synchronized void load() {
		if (this.iconItems == null) {
			log.info("Loading all icons...");
			HashMap<String, IconItem> items = new HashMap<String, IconItem>();
			for (Map.Entry<String, IconDefinition> iconDefinitionEntry : this.iconDefinitions
					.entrySet()) {
				String id = iconDefinitionEntry.getKey();
				IconDefinition iconDefinition = iconDefinitionEntry.getValue();
				IconItem iconItem = new IconItem();
				//
				Map<IconSize, String> paths = iconDefinition.getPaths();
				for (Map.Entry<IconSize, String> pathEntry : paths.entrySet()) {
					IconSize size = pathEntry.getKey();
					String path = pathEntry.getValue();
					Icon iconObject = loadIconObject(path);
					if (iconObject != null) {
						iconItem.iconObjects.put(size, iconObject);
					}
				}
				// Ok
				items.put(id, iconItem);
			}
			log.info("End of the load of all icons.");
			this.iconItems = items;
		}
	}

	/**
	 * This method must be overriden by subclasses in order to load an icon from
	 * a path.
	 * 
	 * @param path
	 *            Path to the icon (could be a resource path but could be
	 *            something else according to the icon manager implementation).
	 * @return Icon
	 */
	protected abstract Icon loadIconObject(String path);

	/**
	 * Sets the list of icon definitions
	 * 
	 * @param iconDefinitions
	 *            List of icon definitions
	 */
	public void setIconDefinitions(Map<String, IconDefinition> iconDefinitions) {
		this.iconDefinitions = iconDefinitions;
	}

	/**
	 * This method must be overriden by subclasses in order to return the URL to
	 * an icon defined by the given path.
	 * 
	 * @param path
	 *            Path to the icon (could be a resource path but could be
	 *            something else according to the icon manager implementation).
	 * @return Icon URL
	 */
	protected abstract URL toURL(String path);

}
