package net.sf.doolin.gui.service.support;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.service.ConfirmResult;
import net.sf.doolin.gui.service.InfoService;
import net.sf.doolin.gui.service.info.DefaultErrorMessageService;
import net.sf.doolin.gui.service.info.ErrorMessageService;
import net.sf.doolin.gui.util.GUIStrings;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * Logger service based on the display of alerts.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class DefaultInfoService implements InfoService {

	private static final Logger log = LoggerFactory.getLogger(DefaultInfoService.class);

	private ErrorMessageService errorMessageService = new DefaultErrorMessageService();

	@Override
	public boolean confirm(ActionContext actionContext, Object messageKey,
			Object... messageParameters) {
		// Message
		String message = GUIStrings.get(messageKey, messageParameters);
		// Title
		String title = GUIStrings.get("InfoService.confirm.title");
		// Displays the box
		int result = JOptionPane.showConfirmDialog(
				actionContext.getComponent(), message, title,
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
		// Result
		return result == JOptionPane.YES_OPTION;
	}

	@Override
	public ConfirmResult confirm3(ActionContext actionContext,
			Object messageKey, Object... messageParameters) {
		// Message
		String message = GUIStrings.get(messageKey, messageParameters);
		// Title
		String title = GUIStrings.get("InfoService.confirm.title");
		// Displays the box
		int result = JOptionPane.showConfirmDialog(
				actionContext.getComponent(), message, title,
				JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
		// Result
		switch (result) {
		case JOptionPane.YES_OPTION:
			return ConfirmResult.YES;
		case JOptionPane.NO_OPTION:
			return ConfirmResult.NO;
		default:
			return ConfirmResult.CANCEL;
		}
	}

	/**
	 * Creates the component for a message
	 * 
	 * @param message
	 *            Message to display
	 * @return Message component
	 */
	protected JComponent createMessageComponent(String message) {
		JTextArea area = new JTextArea(10, 50);
		area.setEditable(false);
		area.setText(message);
		area.setCaretPosition(0);
		area.setWrapStyleWord(true);
		area.setLineWrap(true);
		JScrollPane scrollPane = new JScrollPane(area);
		return scrollPane;
	}

	@Override
	public void error(ActionContext actionContext, Throwable e,
			Object messageKey, Object... messageParameters) {
		// Object list
		List<Object> objectList = new ArrayList<Object>();
		// Message
		String message = GUIStrings.get(messageKey, messageParameters);
		objectList.add(message);
		// Error message
		if (e != null) {
			String errorMessage = getErrorMessage(e);
			objectList.add(createMessageComponent(errorMessage));
			// Logs on console
			log.error("Error", e);
		}
		// Title
		String title = GUIStrings.get("InfoService.error.title");
		// Displays the box
		JOptionPane.showMessageDialog(actionContext.getComponent(), objectList
				.toArray(), title, JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Creates the error message for an exception
	 * 
	 * @param e
	 *            Exception
	 * @return Error message
	 * @see ErrorMessageService
	 */
	protected String getErrorMessage(Throwable e) {
		return getErrorMessageService().getErrorMessage(e);
	}

	public ErrorMessageService getErrorMessageService() {
		return this.errorMessageService;
	}

	@Override
	public void info(ActionContext actionContext, Object messageKey,
			Object... messageParameters) {
		// Message
		String message = GUIStrings.get(messageKey, messageParameters);
		// Title
		String title = GUIStrings.get("InfoService.info.title");
		// Displays the box
		JOptionPane.showMessageDialog(actionContext.getComponent(), message,
				title, JOptionPane.INFORMATION_MESSAGE);
	}

	@Autowired(required = false)
	public void setErrorMessageService(ErrorMessageService errorMessageService) {
		this.errorMessageService = errorMessageService;
	}

	@Override
	public void warning(ActionContext actionContext, Throwable e,
			Object messageKey, Object... messageParameters) {
		// Object list
		List<Object> objectList = new ArrayList<Object>();
		// Message
		String message = GUIStrings.get(messageKey, messageParameters);
		objectList.add(message);
		// Error message
		if (e != null) {
			String errorMessage = getErrorMessage(e);
			objectList.add(createMessageComponent(errorMessage));
			// Logs on console
			log.warn("Warning", e);
		}
		// Title
		String title = GUIStrings.get("InfoService.warning.title");
		// Displays the box
		JOptionPane.showMessageDialog(actionContext.getComponent(), objectList
				.toArray(), title, JOptionPane.WARNING_MESSAGE);
	}
}
