package net.sf.doolin.gui.service.support;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;

/**
 * This implementation saves preferences in a local property file.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class PropertyFileGUIPreferences extends AbstractGUIPreferences {

	private static final Logger log = LoggerFactory
			.getLogger(PropertyFileGUIPreferences.class);

	private String filePath;

	private Properties properties;

	/**
	 * Gets the relative file path to the preferences file.
	 * 
	 * @return Relative file path
	 */
	public String getFilePath() {
		return this.filePath;
	}

	/**
	 * Sets the relative file path to the preferences file (may be local).
	 * 
	 * @param filePath
	 *            Relative file path
	 */
	@Required
	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}

	/**
	 * Lazily initialises the preferences by loading them.
	 * 
	 * @see #load()
	 */
	protected void init() {
		if (this.properties == null) {
			load();
		}
	}

	/**
	 * Actually loads the preferences.
	 */
	protected synchronized void load() {
		if (this.properties == null) {
			Properties p = new Properties();
			File file = new File(this.filePath);
			if (file.exists()) {
				try {
					BufferedInputStream in = new BufferedInputStream(
							new FileInputStream(file));
					try {
						p.load(in);
					} finally {
						in.close();
					}
				} catch (IOException ex) {
					log.error(String.format("Cannot load preferences from %s",
							this.filePath), ex);
				}
			}
			this.properties = p;
		}
	}

	@Override
	public void delete(String key) {
		init();
		this.properties.remove(key);
	}

	@Override
	public Boolean getBoolean(String key, Boolean defaultValue) {
		init();
		String value = getString(key, null);
		if (value != null) {
			return Boolean.valueOf(value);
		} else {
			return defaultValue;
		}
	}

	@Override
	public File getFile(String key) {
		init();
		String value = getString(key, null);
		if (value != null) {
			return new File(value);
		} else {
			return null;
		}
	}

	@Override
	public Integer getInt(String key, Integer defaultValue) {
		init();
		String value = getString(key, null);
		if (value != null) {
			return Integer.valueOf(value, 10);
		} else {
			return defaultValue;
		}
	}

	@Override
	public Long getLong(String key, Long defaultValue) {
		init();
		String value = getString(key, null);
		if (value != null) {
			return Long.valueOf(value);
		} else {
			return defaultValue;
		}
	}

	@Override
	public String getString(String key, String defaultValue) {
		init();
		String value = this.properties.getProperty(key);
		if (value != null) {
			return value;
		} else {
			return defaultValue;
		}
	}

	@Override
	public void save() {
		init();
		File file = new File(this.filePath);
		try {
			BufferedOutputStream out = new BufferedOutputStream(
					new FileOutputStream(file));
			try {
				this.properties.store(out, "Preferences");
			} finally {
				out.close();
			}
		} catch (IOException ex) {
			log.error(String.format("Cannot save preferences in %s",
					this.filePath), ex);
		}
	}

	@Override
	public void setBoolean(String key, Boolean value) {
		init();
		if (value != null) {
			this.properties.put(key, String.valueOf(value));
		} else {
			this.properties.remove(key);
		}
	}

	@Override
	public void setFile(String key, File file) {
		init();
		if (file != null) {
			this.properties.put(key, file.getAbsolutePath());
		} else {
			this.properties.remove(key);
		}
	}

	@Override
	public void setInt(String key, Integer value) {
		init();
		if (value != null) {
			this.properties.put(key, String.valueOf(value));
		} else {
			this.properties.remove(key);
		}
	}

	@Override
	public void setLong(String key, Long value) {
		init();
		if (value != null) {
			this.properties.put(key, String.valueOf(value));
		} else {
			this.properties.remove(key);
		}
	}

	@Override
	public void setString(String key, String value) {
		init();
		if (value != null) {
			this.properties.put(key, value);
		} else {
			this.properties.remove(key);
		}
	}

}
