/*
 * Created on Jul 25, 2007
 */
package net.sf.doolin.gui.swing;

import net.sf.doolin.util.Utils;
import net.sf.jstring.LocalizableException;
import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.LocalDate;
import org.joda.time.LocalDateTime;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * Label provider for a date.
 * 
 * @author Damien Coraboeuf
 */
public class DateLabelInfoProvider implements LabelInfoProvider {

	private String style = null;

	private String timeStyle = null;

	private String format = null;

	/**
	 * Extracts the date from the item.
	 * 
	 * @param item
	 *            Item
	 * @return Date
	 */
	protected Date getDate(Object item) {
		if (item == null) {
			return null;
		} else if (item instanceof Date) {
			return (Date) item;
		} else if (item instanceof Calendar) {
			return ((Calendar) item).getTime();
		} else if (item instanceof DateTime) {
			DateTime dateTime = (DateTime) item;
			return dateTime.toDate();
		} else if (item instanceof LocalDateTime) {
			LocalDateTime dateTime = (LocalDateTime) item;
			return dateTime.toDateTime().toDate();
		} else if (item instanceof LocalDate) {
			LocalDate dateTime = (LocalDate) item;
			return dateTime.toDateTimeAtMidnight().toDate();
		} else {
			throw new LocalizableException("DateLabelnfoProvider.CannotGetDate", item
					.getClass().getName());
		}
	}

	/**
	 * Computes the <code>DataFormat</code> to use.
	 * 
	 * If the <code>format</code> attribute is set, is is used to create a
	 * <code>{@link SimpleDateFormat}</code> instance.
	 * 
	 * If either <code>style</code> or <code>timeStyle</code> attribute is set,
	 * a <code>DateFormat</code> instance is created from the style (see
	 * <code>{@link DateFormat#getDateInstance(int)}</code>,
	 * <code>{@link DateFormat#getDateTimeInstance(int, int)}</code> and
	 * <code>{@link DateFormat#getTimeInstance(int)}</code>).
	 * 
	 * If no attribute is set, a default <code>DateFormat</code> instance is
	 * created (see <code>{@link DateFormat#getDateInstance()}</code>).
	 * 
	 * @return <code>DateFormat</code> instance.
	 */
	protected DateFormat getDateFormat() {
		if (StringUtils.isNotBlank(this.format)) {
			return new SimpleDateFormat(this.format);
		} else if (StringUtils.isNotBlank(this.style)) {
			int formatStyle = (Integer) Utils.getConstant(DateFormat.class,
					this.style);
			if (StringUtils.isNotBlank(this.timeStyle)) {
				int formatTimeStyle = (Integer) Utils.getConstant(
						DateFormat.class, this.style);
				return DateFormat.getDateTimeInstance(formatStyle,
						formatTimeStyle);
			} else {
				return DateFormat.getDateInstance(formatStyle);
			}
		} else if (StringUtils.isNotBlank(this.timeStyle)) {
			int formatTimeStyle = (Integer) Utils.getConstant(DateFormat.class,
					this.timeStyle);
			return DateFormat.getTimeInstance(formatTimeStyle);
		} else {
			return DateFormat.getDateInstance();
		}
	}

	/**
	 * Returns the <code>format</code> property.
	 * 
	 * @return <code>format</code> property.
	 */
	public String getFormat() {
		return this.format;
	}

	public LabelInfo getLabelIcon(Object item) {
		if (item != null) {
			DateFormat dateFormat = getDateFormat();
			Date date = getDate(item);
			String text = dateFormat.format(date);
			return new LabelInfo(text, null);
		} else {
			return new LabelInfo("", null);
		}
	}

	/**
	 * Returns the <code>style</code> property.
	 * 
	 * @return <code>style</code> property.
	 */
	public String getStyle() {
		return this.style;
	}

	/**
	 * Returns the <code>timeStyle</code> property.
	 * 
	 * @return <code>timeStyle</code> property.
	 */
	public String getTimeStyle() {
		return this.timeStyle;
	}

	/**
	 * Sets the <code>format</code> property.
	 * 
	 * @param format
	 *            <code>format</code> property.
	 */
	public void setFormat(String format) {
		this.format = format;
	}

	/**
	 * Sets the <code>style</code> property.
	 * 
	 * @param style
	 *            <code>style</code> property.
	 */
	public void setStyle(String style) {
		this.style = style;
	}

	/**
	 * Sets the <code>timeStyle</code> property.
	 * 
	 * @param timeStyle
	 *            <code>timeStyle</code> property.
	 */
	public void setTimeStyle(String timeStyle) {
		this.timeStyle = timeStyle;
	}

}
