package net.sf.doolin.gui.swing;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.Window;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.table.TableColumn;

import net.sf.doolin.util.unit.Unit;
import net.sf.doolin.util.unit.ValueUnit;

import org.apache.commons.beanutils.ConstructorUtils;

import com.google.common.base.Predicate;

/**
 * Several utility methods for dealing with Swing.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class SwingUtils {

	/**
	 * Checks the class of a component
	 */
	public static class ClassComponentPredicate implements Predicate<Component> {

		private final Class<? extends Component> componentClass;

		/**
		 * Constructor
		 * 
		 * @param componentClass
		 *            Class of the component
		 */
		public ClassComponentPredicate(Class<? extends Component> componentClass) {
			this.componentClass = componentClass;
		}

		@Override
		public boolean apply(Component e) {
			return this.componentClass.isInstance(e);
		}

	}

	/**
	 * Width of the stroke when drawing a shape
	 * 
	 * @see #drawXORShape(JComponent, Rectangle)
	 */
	public static final int XOR_STROKE_WIDTH = 3;

	/**
	 * Copies items of a tool bar to another tool bar
	 * 
	 * @param targetBar
	 *            Target tool bar
	 * @param sourceBar
	 *            Source tool bar
	 */
	public static void copyToolbar(JToolBar targetBar, JToolBar sourceBar) {
		// List of source component
		List<Component> sourceComponents = new ArrayList<Component>();
		int count = sourceBar.getComponentCount();
		for (int i = 0; i < count; i++) {
			Component c = sourceBar.getComponent(i);
			sourceComponents.add(c);
		}
		// Clean
		targetBar.removeAll();
		// Copy
		for (Component c : sourceComponents) {
			targetBar.add(c);
		}
	}

	/**
	 * Creates an instance of a frame or a dialog
	 * 
	 * @param windowClass
	 *            Class of {@link Window} to create
	 * @param parent
	 *            Parent component (can be <code>null</code>)
	 * @param <T>
	 *            Type of window
	 * @return Window
	 */
	public static <T> T createWindow(Class<T> windowClass, Component parent) {
		try {
			if (parent == null) {
				return windowClass.newInstance();
			} else {
				Window parentWindow = SwingUtilities.getWindowAncestor(parent);
				if (parentWindow != null) {
					@SuppressWarnings("unchecked")
					T window = (T) ConstructorUtils.invokeConstructor(
							windowClass, parentWindow);
					return window;
				} else {
					return windowClass.newInstance();
				}
			}
		} catch (Exception ex) {
			throw new RuntimeException("Cannot instantiate instance of "
					+ windowClass.getName());
		}
	}

	/**
	 * Draws a shape for a component
	 * 
	 * @param component
	 *            Component to draw the shape into
	 * @param shape
	 *            Shape to draw
	 * @see #drawXORShape(JComponent, Rectangle)
	 */
	public static void drawGrayShape(JComponent component, Rectangle shape) {
		Rectangle oldShape = (Rectangle) component
				.getClientProperty("grayShape");
		if (oldShape != null && (shape == null || !oldShape.equals(shape))) {
			drawXORShape(component, oldShape);
		}
		// New shape
		if (shape != null) {
			drawXORShape(component, shape);
			component.putClientProperty("grayShape", shape);
		}
		// No shape
		else {
			component.putClientProperty("grayShape", null);
		}
	}

	/**
	 * Draws an XOR shape in a component
	 * 
	 * @param component
	 *            Component to draw the shape into
	 * @param shape
	 *            Shape to draw
	 */
	public static void drawXORShape(JComponent component, Rectangle shape) {
		Rectangle bounds = new Rectangle(shape);
		bounds.width -= XOR_STROKE_WIDTH;
		bounds.height -= XOR_STROKE_WIDTH;
		Graphics2D g = (Graphics2D) component.getGraphics();
		Stroke oldStroke = g.getStroke();
		Color oldColor = g.getColor();
		try {
			g.setXORMode(Color.WHITE);
			g.setColor(Color.BLACK);
			g.setStroke(new BasicStroke(XOR_STROKE_WIDTH));
			g.draw(bounds);
		} finally {
			g.setPaintMode();
			g.setColor(oldColor);
			g.setStroke(oldStroke);
		}
	}

	/**
	 * Recursively descends the parent container, applies the predicate to each
	 * component and returns the first one the predicate has returned
	 * <code>true</code> for.
	 * 
	 * @param parent
	 *            Parent container
	 * @param predicate
	 *            Test predicate
	 * @return Found component or <code>null</code>
	 */
	public static Component findComponentInTree(Container parent,
			Predicate<Component> predicate) {
		int count = parent.getComponentCount();
		for (int i = 0; i < count; i++) {
			Component component = parent.getComponent(i);
			if (predicate.apply(component)) {
				return component;
			} else if (component instanceof Container) {
				Component result = findComponentInTree((Container) component,
						predicate);
				if (result != null) {
					return result;
				}
			}
		}
		return null;
	}

	/**
	 * Set the width of a column
	 * 
	 * @param table
	 *            Table
	 * @param column
	 *            Column to set
	 * @param width
	 *            Width to set
	 * @param resizable
	 *            <code>true</code> if the column can be resized
	 */
	public static void setTableColumnWidth(JTable table, int column,
			ValueUnit width, boolean resizable) {
		int tableWidth = table.getWidth();
		int screenWidth = width.convert(Unit.PIXEL, tableWidth).getValue()
				.intValue();
		TableColumn tableColumn = table.getColumnModel().getColumn(column);
		if (resizable) {
			tableColumn.setResizable(true);
			tableColumn.setMinWidth(0);
			tableColumn.setMaxWidth(800);
		} else {
			tableColumn.setMinWidth(screenWidth);
			tableColumn.setMaxWidth(screenWidth);
			tableColumn.setResizable(false);
		}
		tableColumn.setPreferredWidth(screenWidth);
	}

	/**
	 * Creates an image from an icon. So far, only instances of
	 * {@link javax.swing.ImageIcon} are accepted.
	 * 
	 * @param icon
	 *            Icon
	 * @return Image or <code>null</code> if not appliable.
	 */
	public static Image toImage(Icon icon) {
		if (icon instanceof ImageIcon) {
			ImageIcon imageIcon = (ImageIcon) icon;
			return imageIcon.getImage();
		} else {
			return null;
		}
	}

	private SwingUtils() {
	}
}
