package net.sf.doolin.gui.validation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * This class defines the result for a validation. It is mainly a list of
 * {@link GUIValidationMessage} instances.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class GUIValidation {

	/**
	 * Empty (valid) validation object.
	 */
	public static final GUIValidation EMPTY = new GUIValidation();

	private List<GUIValidationMessage> messages = new ArrayList<GUIValidationMessage>();

	/**
	 * Adds a validation message
	 * 
	 * @param message
	 *            Message to add
	 */
	public void addMessage(GUIValidationMessage message) {
		this.messages.add(message);
	}

	/**
	 * Adds a validation message
	 * 
	 * @param key
	 *            Key of the field
	 * @param msgKey
	 *            Text key for the displayed message
	 * @param params
	 *            Parameters for the displayed message
	 * @see GUIValidationMessage#GUIValidationMessage(Object, String, Object...)
	 */
	public void addMessage(Object key, String msgKey, Object... params) {
		addMessage(new GUIValidationMessage(key, msgKey, params));
	}

	/**
	 * Gets the list of messages
	 * 
	 * @return List of messages
	 */
	public List<GUIValidationMessage> getMessages() {
		return this.messages;
	}

	/**
	 * Checks if this {@link GUIValidation} object is valid. It is valid if the
	 * list of messages is empty.
	 * 
	 * @return <code>true</code> if valid
	 */
	public boolean isValid() {
		return this.messages.isEmpty();
	}

	/**
	 * Checks if this {@link GUIValidation} object has errors.
	 * 
	 * @return <code>true</code> if this validation has errors
	 * @see #isValid()
	 */
	public boolean hasErrors() {
		return !isValid();
	}

	/**
	 * Merge the <code>subResults</code> to this validation object.
	 * 
	 * @param subResults
	 *            {@link GUIValidation} to merge
	 */
	public void addAllFrom(GUIValidation subResults) {
		if (subResults != null) {
			this.messages.addAll(subResults.getMessages());
		}
	}

	/**
	 * Returns an unmodifiable version of the given {@link GUIValidation}
	 * instance
	 * 
	 * @param source
	 *            Source object
	 * @return Unmodifiable object
	 */
	public static GUIValidation unmodifiable(GUIValidation source) {
		GUIValidation result = new GUIValidation();
		result.messages = Collections.unmodifiableList(source.messages);
		return result;
	}

	/**
	 * Returns a map of validations objects, indexed by field keys.
	 * 
	 * @return Map of validations objects, indexed by field keys
	 */
	public Map<Object, GUIValidation> keyMap() {
		Map<Object, GUIValidation> map = new HashMap<Object, GUIValidation>();
		for (GUIValidationMessage message : this.messages) {
			Object key = message.key();
			GUIValidation validation = map.get(key);
			if (validation == null) {
				validation = new GUIValidation();
				map.put(key, validation);
			}
			validation.addMessage(message);
		}
		return map;
	}
}
