package net.sf.doolin.gui.window.support;

import java.awt.BorderLayout;
import java.awt.Window;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.path.ActionBarContext;
import net.sf.doolin.gui.action.path.ActionContextPathList;
import net.sf.doolin.gui.action.path.DefaultActionContextPathList;
import net.sf.doolin.gui.action.path.toolbar.ToolBar;
import net.sf.doolin.gui.swing.DockLayout;
import net.sf.doolin.gui.swing.SwingUtils;
import net.sf.doolin.util.Pair;

/**
 * Window implementation that allows the management of tool bars and menu bars.
 * It is based on the {@link JRootPane} of the window implementation. This pane
 * contains a {@link #getRootPanel() root panel} based on a {@link DockLayout}
 * and contains as its main content a {@link #getWindowPanel() window panel},
 * used to hold the actual content of the window.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of data for the window
 */
public abstract class AbstractRootPaneGUIWindow<B> extends AbstractGUIWindow<B> {

	private JPanel rootPanel;
	private JPanel windowPanel;
	private Map<String, Pair<ActionContext, JToolBar>> toolBars = new HashMap<String, Pair<ActionContext, JToolBar>>();

	/**
	 * List of menu bars and their context
	 */
	protected final Map<String, ActionBarContext> menuBarContextMap = new LinkedHashMap<String, ActionBarContext>(
			2);

	@Override
	protected void build() {
		// Root panel
		this.rootPanel = new JPanel(new DockLayout());
		// Creates the window panel
		this.windowPanel = new JPanel();
		this.rootPanel.add(this.windowPanel, BorderLayout.CENTER);
	}

	/**
	 * Gets the root panel for this window.
	 * 
	 * @return Root panel
	 */
	public JPanel getRootPanel() {
		return this.rootPanel;
	}

	@Override
	public JPanel getWindowPanel() {
		return this.windowPanel;
	}

	@Override
	public void installMenuBar(String id, ActionBarContext menuBar) {
		// Checks already install menu bar
		ActionBarContext existing = this.menuBarContextMap.get(id);
		// Same context
		if (existing != null && existing.getActionContext() == menuBar.getActionContext()) {
			// Does nothing
			return;
		}
		// Registers the new context
		this.menuBarContextMap.put(id, menuBar);
		// Set-up of the menu bar
		installMenuBar();
	}

	@Override
	public void uninstallMenuBar(String id) {
		// Unregisters the new context
		if (this.menuBarContextMap.containsKey(id)) {
			this.menuBarContextMap.remove(id);
			// Set-up of the menu bar
			installMenuBar();
		}
	}

	@Override
	public void installToolBar(ToolBar toolBar, ActionContext actionContext) {
		// Bar name
		String name = toolBar.getName();
		// Gets any existing tool bar
		Pair<ActionContext, JToolBar> pair = this.toolBars.get(name);
		JToolBar existingJToolBar = null;
		// Checks for replacement
		if (pair != null) {
			if (actionContext == pair.getFirstValue()) {
				return;
			}
			existingJToolBar = pair.getSecondValue();
		}
		// Creates the tool bar
		ActionBarContext actionBarContext = new ActionBarContext(toolBar
				.getActionPathList(), actionContext);
		actionBarContext.setProperty(ToolBar.FLOATABLE, toolBar.isFloatable());
		JToolBar jtoolBar = getWindowDescriptor().getActionFactory()
				.createToolBar(actionBarContext);
		jtoolBar.setName(name);
		// Updates the existing tool bar
		if (existingJToolBar != null) {
			SwingUtils.copyToolbar(existingJToolBar, jtoolBar);
		} else {
			Object constraint = toolBar.getPosition().getBorderConstraint();
			this.rootPanel.add(jtoolBar, constraint);
			this.toolBars.put(name, new Pair<ActionContext, JToolBar>(
					actionContext, jtoolBar));
		}
	}

	@Override
	public void uninstallToolBar(String id) {
		Pair<ActionContext, JToolBar> pair = this.toolBars.get(id);
		if (pair != null) {
			this.toolBars.remove(id);
			JToolBar existingToolBar = pair.getSecondValue();
			// Closes any id tool bar
			Window thisWindow = SwingUtilities
					.getWindowAncestor(this.rootPanel);
			Window window = SwingUtilities.getWindowAncestor(existingToolBar);
			if (window != thisWindow) {
				window.setVisible(false);
			}
			// Removes the tool bar from the frame
			this.rootPanel.remove(existingToolBar);
			this.rootPanel.repaint();
		}
	}

	/**
	 * Installs the menu bar
	 */
	protected void installMenuBar() {
		ActionContextPathList menuBarContext = new DefaultActionContextPathList(
				this.menuBarContextMap);
		JMenuBar jmenuBar = getWindowDescriptor().getActionFactory()
				.createMenuBar(menuBarContext);
		this.windowPanel.getRootPane().setJMenuBar(jmenuBar);
	}

}