package net.sf.doolin.gui.wizard.descriptor;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.descriptor.GUIViewDescriptor;
import net.sf.doolin.gui.window.GUIWindow;
import net.sf.doolin.gui.window.GUIWindowFactory;
import net.sf.doolin.gui.window.descriptor.AbstractGUIWindowDescriptor;
import net.sf.doolin.gui.window.support.DefaultIWindowFactory;
import net.sf.doolin.gui.window.support.IWindowFactory;
import net.sf.doolin.gui.wizard.rule.WizardRules;
import net.sf.doolin.gui.wizard.state.WizardState;
import net.sf.doolin.gui.wizard.support.WizardWindowFactory;
import net.sf.jstring.LocalizableException;
import org.apache.commons.lang.Validate;
import org.springframework.beans.factory.annotation.Required;

import javax.swing.*;
import java.awt.*;
import java.util.Map;

/**
 * Window descriptor for a wizard.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of bean for the wizard
 */
public class WizardDescriptor<B> extends AbstractGUIWindowDescriptor<B> {

	private Map<String, GUIViewDescriptor<B>> viewMap;
	private String initialView;
	private WizardRules<B> wizardRules;
	private GUIView<B> currentView;
	private IWindowFactory iwindowFactory = new DefaultIWindowFactory();
	private boolean modal = true;

	/**
	 * Constructor that fixes the window factory to {@link WizardWindowFactory}.
	 */
	public WizardDescriptor() {
		// Sets the wizard window factory
		this(new WizardWindowFactory());
	}

	/**
	 * Constructor that uses a custom version of {@link WizardWindowFactory}.
	 */
	public WizardDescriptor(WizardWindowFactory wizardWindowFactory) {
		// Sets the wizard window factory
		setWindowFactory(wizardWindowFactory);
	}

	@Override
	protected void arrangeViews(final GUIWindow<B> window) {
		// Wizard state
		final WizardState<B> state = new WizardState<B>(window.getWindowData(),
				this.wizardRules, this.initialView);
		window.getActionContext().setContext(state);
		// Initial view
		setViewId(window, this.initialView);
		// Listens to state changes
		PropertyChangeSupport.subscribe(window.getActionContext()
				.getSubscriberValidator(), state, WizardState.CURRENT_VIEW,
				new Runnable() {

					@Override
					public void run() {
						setViewId(window, state.getCurrentView());
					}
				});
	}

	@Override
	protected void arrangeWindow(GUIWindow<B> window) {
		// Gets the window content
		JPanel windowPanel = window.getWindowPanel();
		// Sets its layout
		windowPanel.setLayout(new BorderLayout());
	}

	/**
	 * Gets the initial view for this wizard
	 * 
	 * @return Initial view name
	 */
	public String getInitialView() {
		return this.initialView;
	}

	/**
	 * Gets the factory for the wizard window implementation
	 * 
	 * @return Window implementation factory
	 */
	public IWindowFactory getIwindowFactory() {
		return this.iwindowFactory;
	}

	/**
	 * Gets the actual view descriptor from a view name
	 * 
	 * @param viewId
	 *            View name
	 * @return View descriptor
	 */
	protected GUIViewDescriptor<B> getViewDescriptor(String viewId) {
		GUIViewDescriptor<B> viewDescriptor = this.viewMap.get(viewId);
		if (viewDescriptor != null) {
			return viewDescriptor;
		} else {
			throw new LocalizableException("WizardDescriptor.ViewNotFound", viewId);
		}
	}

	/**
	 * Gets the mapping between view names and actual
	 * {@linkplain GUIViewDescriptor view descriptors}.
	 * 
	 * @return Mapping between view names and actual view descriptors
	 */
	public Map<String, GUIViewDescriptor<B>> getViewMap() {
		return this.viewMap;
	}

	/**
	 * Gets the navigation rules for this wizard
	 * 
	 * @return Navigation rules
	 */
	public WizardRules<B> getWizardRules() {
		return this.wizardRules;
	}

	/**
	 * Checks if the wizard must be executed in modal mode or not.
	 * 
	 * @return Modal mode
	 */
	public boolean isModal() {
		return this.modal;
	}

	/**
	 * Always return <code>null</code> since a wizard cannot hold views.
	 */
	@Override
	public <V> GUIView<V> openView(GUIWindow<B> parentWindow,
			GUIViewDescriptor<V> viewDescriptor, V viewData) {
		return null;
	}

	/**
	 * Sets the initial view name for this wizard.
	 * 
	 * @param initialView
	 *            Initial view name
	 */
	@Required
	public void setInitialView(String initialView) {
		this.initialView = initialView;
	}

	/**
	 * Sets the window implementation factory for this wizard. It is set by
	 * default to {@link DefaultIWindowFactory}.
	 * 
	 * @param iwindowFactory
	 *            Window implementation factory
	 */
	public void setIwindowFactory(IWindowFactory iwindowFactory) {
		this.iwindowFactory = iwindowFactory;
	}

	/**
	 * Sets the modal mode of the wizard
	 * 
	 * @param modal
	 *            Modal mode
	 */
	public void setModal(boolean modal) {
		this.modal = modal;
	}

	/**
	 * Navigates to a new view. Any previous view is closed and the new view is
	 * created from scratch before being displayed in the wizard.
	 * 
	 * @param window
	 *            Wizard window
	 * @param viewId
	 *            View to navigate to.
	 */
	protected void setViewId(GUIWindow<B> window, String viewId) {
		// Closes the previous view
		if (this.currentView != null) {
			this.currentView.close();
			this.currentView = null;
		}
		// Gets the view
		GUIViewDescriptor<B> viewDescriptor = getViewDescriptor(viewId);
		// Creates the view
		GUIView<B> view = viewDescriptor.createView(window, window
				.getWindowData());
		// View component
		JComponent viewComponent = initViewComponent(window, view, null, null);
		// Gets the window content
		JPanel windowPanel = window.getWindowPanel();
		// Positions the view in the window
		windowPanel.removeAll();
		windowPanel.add(viewComponent, BorderLayout.CENTER);
		windowPanel.validate();
		windowPanel.repaint();
		// OK
		this.currentView = view;
		window.addView(this.currentView);
	}

	/**
	 * Sets the mapping between view names and actual
	 * {@linkplain GUIViewDescriptor view descriptors}.
	 * 
	 * @param viewMap
	 *            Mapping between view names and actual view descriptors
	 */
	@Required
	public void setViewMap(Map<String, GUIViewDescriptor<B>> viewMap) {
		this.viewMap = viewMap;
	}

	/**
	 * Sets the window factory for this wizard. It <i>must</i> be a sub-class of
	 * the {@link WizardWindowFactory}.
	 */
	@Override
	public void setWindowFactory(GUIWindowFactory windowFactory) {
		Validate.isTrue(windowFactory instanceof WizardWindowFactory,
				"The window factory must be a WizardWindowFactory");
		super.setWindowFactory(windowFactory);
	}

	/**
	 * Sets the navigation rules for this wizard.
	 * 
	 * @param wizardRules
	 *            Navigation rules
	 */
	@Required
	public void setWizardRules(WizardRules<B> wizardRules) {
		this.wizardRules = wizardRules;
	}

}
